﻿using Newtonsoft.Json.Linq;
using System;
using System.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure.Repository;
using Wicture.DbRESTFul.Rpc.Client;

namespace Wicture.DbRESTFul.Samples.MySqlService.Repositories
{
    public class CommonRepository : DbRESTFulRepository
    {
        private readonly IRpcClient _rpcClient;
        public CommonRepository(IRpcClient rpcClient)
        {
            _rpcClient = rpcClient;
        }
        /// <summary>
        /// Async way method.
        /// </summary>
        public async Task<object> ListAreas()
        {
            return await Task.FromResult(new { id = 1, name = "上海" }); 
        }

        /// <summary>
        /// Sync way method.
        /// </summary>
        public object HelpFiles()
        {
            return new { filePath = "test" }; 
        }

        /// <summary>
        /// InvokeAsync and InvokeSingleOrDefaultAsync with shared connection.
        /// </summary>
        public async Task<object> ListAllBrand()
        {
            using var conn = ConnectionManager.GetConnection();
            var brands = await base.InvokeAsync<Brand>("List_brand_all", new { }, conn);
            string comingSoon = await base.InvokeSingleOrDefaultAsync<string>("Get_ComingSoon", new { }, conn);
            string brandNew = await base.InvokeSingleOrDefaultAsync<string>("Get_BrandNew", new { }, conn);

            if (!string.IsNullOrEmpty(comingSoon))
            {
                var brandCodes = comingSoon.Split(',');
                brands.ForEach(t => { t.comingSoon = brandCodes.Contains(t.brandCode); });
            }
            if (!string.IsNullOrEmpty(brandNew))
            {
                var brandNews = brandNew.Split(',');
                brands.ForEach(t => { t.brandNew = brandNews.Contains(t.brandCode); });
            }
            return brands;
        }


        /// <summary>
        /// QuerySingleOrDefaultAsync and QuerySingleOrDefaultAsync with shared connection.
        /// </summary>
        public async Task<object> ListFQA(JToken param)
        {
            using var conn = ConnectionManager.GetConnection();
            var csi = ResourceManager.Csis["List_Common_Question"];
            var questions = await base.QueryAsync<Question>(csi.code.ToString(), param, conn);

            csi = ResourceManager.Csis["Get_CommonQuestion_Detail"];
            var question1 = await base.QuerySingleOrDefaultAsync<Question>(csi.code.ToString(), new { id = 26 }, conn);
            var question2 = await base.QuerySingleOrDefaultAsync<Question>(csi.code.ToString(), new { id = 0 }, conn);

            return new { questions, question1, question2 };
        }

        public UserIdentity GetUserInfo(string username, string password)
        {
            if (username == "Dawson" && password == "123456")
            {
                return new UserIdentity { From = "Unknown", Id = "0", Name = username };
            }

            throw new LogicalException("Invalid username and password.");
        }

        public async Task<object> GetQuestionDetail(JToken param)
        {
            var response = await _rpcClient.InvokeRpcAsync(Context, "Wicture.DbRESTFul.Samples.MySqlService", "Get_Question_Detail", param);
            var result = response.GetData<JObject>();

            result["Identity"] = Context.Identity.Name;

            return result;
        }
    }
}
