using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Newtonsoft.Json.Linq;
using System;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Cache;
using Wicture.DbRESTFul.Configuration;
using Wicture.DbRESTFul.Resources.Api;
using Wicture.DbRESTFul.Resources.Csi;

namespace Wicture.DbRESTFul.Core.Test
{
    [TestClass]
    public class ConfigurationTest
    {
        [TestMethod]
        public void TypeTest()
        {
            var t1 = typeof(Task<int>);
            var t2 = typeof(Task);
            var t3 = typeof(CsiModel);
            var t4 = typeof(Task<CsiModel>);

            Task<object> m = Convert(Task.FromResult(new CsiModel()));

            Assert.AreEqual(t1.IsGenericType, true);
        }

        private static async Task<object> Convert(Task task)
        {
            await task;
            var voidTaskType = typeof(Task<>).MakeGenericType(Type.GetType("System.Threading.Tasks.VoidTaskResult"));
            if (voidTaskType.IsAssignableFrom(task.GetType()))
                throw new InvalidOperationException("Task does not have a return value (" + task.GetType().ToString() + ")");
            var property = task.GetType().GetProperty("Result", BindingFlags.Public | BindingFlags.Instance);
            if (property == null)
                throw new InvalidOperationException("Task does not have a return value (" + task.GetType().ToString() + ")");
            return property.GetValue(task);
        }



        [TestMethod]
        public void CsiModelTest()
        {
            var csiModel = new CsiModel() { code = new JValue("test") };
            var infos = csiModel.ListResultSetInfo();
            Assert.AreEqual(infos.Count, 1);
            Assert.AreEqual(infos.FirstOrDefault().MultipleRecords, true);
            Assert.AreEqual(infos.FirstOrDefault().Sql, "test");

            csiModel = new CsiModel() { code = new JValue("test"), resultSet = "S" };
            infos = csiModel.ListResultSetInfo();
            Assert.AreEqual(infos.Count, 1);
            Assert.AreEqual(infos.FirstOrDefault().MultipleRecords, false);
            Assert.AreEqual(infos.FirstOrDefault().Sql, "test");


            csiModel = new CsiModel() { code = JObject.FromObject(new { a = "sql1", b = "sql2" }), resultSet = "S" };
            infos = csiModel.ListResultSetInfo();
            Assert.AreEqual(infos.Count, 2);
            Assert.AreEqual(infos.FirstOrDefault().MultipleRecords, false);
            Assert.AreEqual(infos.FirstOrDefault().Sql, "sql1");

            Assert.AreEqual(infos.LastOrDefault().MultipleRecords, true);
            Assert.AreEqual(infos.LastOrDefault().Sql, "sql2");
        }

        [TestMethod]
        public void ObjectSectionConfigTest()
        {
            //ConfigurationManager.Load();

            //var apiConfig = ConfigurationManager.GetSectionConfig<ApiSectionConfig>();

            //Assert.AreEqual(apiConfig.SectionName, "API");
            //Assert.AreEqual(apiConfig.ShowFullException, true);
            //Assert.AreEqual(apiConfig.Path, "API/");
        }


        [TestMethod]
        public void ListSectionConfigTest()
        {
            //ConfigurationManager.Load();

            //var config = ConfigurationManager.GetSectionConfig<CacheSectionConfig>();

            //Assert.AreEqual(config.SectionName, "CacheConfigs");
            //Assert.AreEqual(config.Items.Count, 2);
            //Assert.AreEqual(config.Items[0].Type, "redis");
            //Assert.AreEqual(config.Items[1].Type, "memcached");
        }
    }
}
