﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure.Repository;

namespace Wicture.DbRESTFul.PMT.Controllers
{
    [Route("dbrestful/devtool/api")]
    public class ApiSyncController : Controller
    {
        private static readonly List<string> ObjectProperties = new List<string> { "cache", "implementation", "parameter", "result", "mock" };
        private readonly ILogger _logger;
        private readonly DbRESTFulRepository _repository;

        public ApiSyncController(ILoggerFactory loggerFactory, DbRESTFulRepository repository) 
        {
            _logger = loggerFactory.CreateLogger("PMT");
            _repository = repository;
        }

        [HttpGet]
        [Route("modules")]
        public async Task<object> ListModules(string projectName)
        {
            return await _repository.InvokeAsync("ListModulesByProjectName", new { projectName });
        }

        [HttpGet]
        [Route("sync")]
        public async Task<object> SyncAllConfiguredAPIs(int projectId, int? moduleId)
        {
            if (projectId < 1) throw new ArgumentException(nameof(projectId));

            var param = moduleId != null ? JObject.FromObject(new { projectId, moduleId }) : JObject.FromObject(new { projectId });
            IEnumerable<dynamic> data = await _repository.InvokeAsync("ListApiByModules", param);

            var result = new JObject();

            foreach (IEnumerable<dynamic> items in data.GroupBy(i => i.module))
            {
                var module = items.FirstOrDefault().module ?? "other";
                result[module] = FillAPIsToFile(items.ToArray());
            }
            
            return result;
        }

        private static JArray FillAPIsToFile(params dynamic[] rawApis)
        {
            var apis = new JArray();
            foreach (var item in rawApis)
            {
                JObject ca = JObject.FromObject(item);
                ca["implemented"] = ca.Value<bool?>("implemented") ?? false;
                ca["useAbsoluteUrl"] = ca.Value<bool?>("useAbsoluteUrl") ?? false;
                ca["allowAnonymous"] = ca.Value<bool?>("allowAnonymous") ?? false;
                ObjectProperties.ForEach(p => ca[p] = JToken.Parse(ca.Value<string>(p) ?? "{}"));
                apis.Add(ca);
            }

            return apis;
        }
    }
}
