﻿using Newtonsoft.Json.Linq;
using System;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure.Repository;
using Wicture.DbRESTFul.Resources.Csi;

namespace Wicture.DbRESTFul.PMT.Repositories
{
    public class CSIRepository : DbRESTFulRepository
    {
        public async Task<object> ExecuteCode(JToken param)
        {
            var projectId = param.Value<int>("projectId");
            if (projectId <= 0) throw new ArgumentException("No projectId specified.", nameof(projectId));

            var pmtConnection = ConnectionManager.DbConnection;

            try
            {
                var project = await base.InvokeAsync("GetProject", new { id = projectId });
                var dbtype = (DatabaseType)project.databaseType;

                ConnectionManager.DbConnection = BuildConnection(project);

                var parameters = JToken.Parse(param.Value<string>("params"));
                return await base.QueryAsync<dynamic>(param.Value<string>("code"), parameters);
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                ConnectionManager.DbConnection = pmtConnection;
            }
        }

        private DbConnection BuildConnection(dynamic project)
        {
            return new DbConnection
            {
                DatabaseType = (DatabaseType)project.databaseType,
                ReadConnectionString = project.connectionString,
                WriteConnectionString = project.connectionString,
            };
        }
    }
}
