﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure;
using Wicture.DbRESTFul.Infrastructure.Repository;
using Wicture.DbRESTFul.Resources.Csi;

namespace Wicture.DbRESTFul.PMT.Repositories
{
    public class ImportRepository : DbRESTFulRepository
    {
        public async Task<object> ImportCSI(JToken param)
        {
            var httpContext = (Context as HttpApiExecutionContext).HttpContext;

            var projectId = param.Value<int>("projectId");
            if (projectId <= 0) throw new ArgumentException("No projectId specified.", nameof(projectId));
            if (httpContext.Request.Form.Files.Count == 0) throw new Exception("No file uploaded.");

            int total = 0;
            int replace = 0;
            
            IEnumerable<dynamic> modules = await base.InvokeAsync("ListModulesForProject", new { projectId });

            foreach (var file in httpContext.Request.Form.Files)
            {
                using (StreamReader reader = new StreamReader(file.OpenReadStream()))
                {
                    var text = reader.ReadToEnd();
                    var csiList = JsonConvert.DeserializeObject<List<CsiModel>>(text);

                    foreach (var api in csiList)
                    {
                        IEnumerable<dynamic> checkResult = await base.InvokeAsync("CheckCSINameExsits", new { projectId, api.name });
                        if (checkResult.Any())
                        {
                            replace++;
                            await base.InvokeAsync("DeleteCSI", new { ids = checkResult.Select(d => d.id).OfType<int>().ToList() });
                        }
                    }

                    await base.InvokeAsync("ImportCSIs", csiList.Select(csi => BeParameterized(projectId, csi, modules)));
                    total += csiList.Count;
                }
            }

            return new { total, replace };
        }

        private object BeParameterized(int projectId, CsiModel csi, IEnumerable<dynamic> modules)
        {
            int? moduleId = modules == null ? null : modules.FirstOrDefault(a => a.name == csi.module)?.id;
            var code = csi.code is JValue ? csi.code.ToString() : csi.code.ToString(Formatting.None);

            return new
            {
                projectId,
                csi.name,
                module = csi.module ?? "default",
                moduleId,
                code,
                rawCode = code,
                csi.resultSet,
                csi.queryOnly,
                csi.requiredTransaction,
                middleWares = JsonConvert.SerializeObject(csi.middlewares),
                updatedBy = Context.Identity.Id,
                createdBy = Context.Identity.Id
            };
        }
    }
}
