﻿using Autofac;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using System.Threading;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Authentication;
using Wicture.DbRESTFul.PMT.Repositories;

namespace Wicture.DbRESTFul.PMT.Service
{
    public class UserIdentifier : IUserIdentifier
    {
        private ILogger _logger;

        public UserIdentifier(ILoggerFactory loggerFactory)
        {
            _logger = loggerFactory.CreateLogger("DbRESTFulApi");
        }

        public async Task AuthorizeAsync(ExecutionContext context)
        {
            _logger.LogInformation($"Authorize {context.Api.name} access for {context.Identity.Name}.");
            context.Identity.IsAuthenticated = true;
            await Task.CompletedTask;
        }

        public async Task<UserIdentity> IdentifyAsync(string username, string password, JToken param, ExecutionContext context = null)
        {
            _logger.LogDebug($"Login for `{username}` with `{password}`.");

            using (var scope = AutofacContainer.Container.BeginLifetimeScope())
            {
                var repository = scope.Resolve<UserRepository>();
                return await repository.GetUserIdentity(username, password);
            }
        }
    }
}
