﻿using Newtonsoft.Json.Linq;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Wicture.DbRESTFul.PMT.SpecGenerators
{
    partial class DbSpecGenerator
    {
        private readonly DbSpecSource source;
        public DbSpecGenerator(JObject config)
        {
            List<TableSpec> tables = GetTables(config.Value<string>("connectionString"), config.Value<JArray>("tables"),config.Value<int>("projectId"));

            source = new DbSpecSource
            {
                moduleName = config.Value<string>("moduleName"),
                projectName = config.Value<string>("projectName"),
                tables = tables
            };
        }

        private List<TableSpec> GetTables(string connectionString, JArray tables,int projectId)
        {
            SpecHelper manager = new SpecHelper(connectionString,projectId);
            var result = new List<TableSpec>();

            manager.ListTables().ForEach(t =>
            {
                var item = tables.OfType<JObject>().FirstOrDefault(i => i.Value<string>("name") == t.name);
                if (item != null)
                {
                    t.title = item.Value<string>("title");
                    t.title = string.IsNullOrEmpty(t.title) ? t.name : t.title;
                    result.Add(t);
                }
            });

            return result;
        }

        public string Generate(string path)
        {
            var fileName = string.Format(@"{0}/{1}/Database/{2}.html", path, source.projectName, source.moduleName);

            if (!Directory.Exists(Path.GetDirectoryName(fileName)))
            {
                Directory.CreateDirectory(Path.GetDirectoryName(fileName));
            }

            var text = this.TransformText();
            File.WriteAllText(fileName, text);

            return fileName;
        }
    }
}