﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Authentication
{
    public class DefaultUserIdentifier : IUserIdentifier
    {
        public const string AuthenticationFailedCode = "402";

        private ILogger _logger;

        public DefaultUserIdentifier(ILoggerFactory loggerFactory)
        {
            _logger = loggerFactory.CreateLogger("DbRESTFulApi");
        }

        public async Task AuthorizeAsync(ExecutionContext context)
        {
            // Ignored.
            _logger.LogInformation($"Authorize {context.Api.name} access for {context.Identity.Name}.");
            context.Identity.IsAuthenticated = true;
            await Task.CompletedTask;
        }

        public async Task<UserIdentity> IdentifyAsync(string username, string password, JToken tag, ExecutionContext execContext)
        {
            _logger.LogDebug($"Login for `{username}` with `{password}`.");

            var identity = new UserIdentity { From = "Unknown", Id = "0", Name = username };
            return await Task.FromResult(identity);
        }
    }
}
