﻿using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Text;
using Wicture.DbRESTFul.Configuration;
using Wicture.DbRESTFul.Resources.Api;

namespace Wicture.DbRESTFul.Authentication
{
    public class TokenOptions : ObjectSectionConfigBase<TokenOptions>
    {
        public const int StartIndex = 7;

        public override string SectionName => "TokenOptions";

        public string TokenRequestPath { get; set; }
        public string RefreshTokenRequestPath { get; set; }

        public string Issuer { get; set; }

        public string Audience { get; set; }

        public string SecretKey { get; set; }

        public bool UseJwtCookie { get; set; }

        public string CookieName { get; set; }

        public bool ForceCookie { get; set; } = false;

        public TimeSpan Expiration { get; set; }

        public Schema[] LoginSchema { get; set; }

        private TokenValidationParameters validationParameters;
        public TokenValidationParameters ValidationParameters
        {
            get
            {
                if (validationParameters == null)
                {
                    validationParameters = new TokenValidationParameters
                    {
                        ValidateIssuerSigningKey = true,
                        IssuerSigningKey = SigningCredentials.Key,

                        ValidateIssuer = false,
                        ValidIssuer = Issuer,

                        ValidateAudience = true,
                        ValidAudience = Audience,

                        ValidateLifetime = true,

                        ClockSkew = TimeSpan.Zero,
                        AuthenticationType = JwtBearerDefaults.AuthenticationScheme
                    };
                }

                return validationParameters;
            }
        }

        private SigningCredentials signingCredentials;
        public SigningCredentials SigningCredentials
        {
            get
            {
                if (signingCredentials == null)
                {
                    var signingKey = new SymmetricSecurityKey(Encoding.ASCII.GetBytes(SecretKey));
                    signingCredentials = new SigningCredentials(signingKey, SecurityAlgorithms.HmacSha256);
                }

                return signingCredentials;
            }
        }
    }
}
