﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Cache
{
    public class DefaultCacheService : ICacheService
    {
        public DefaultCacheService(IEnumerable<ICacheProvider> cacheProviders)
        {
            CacheProviders = cacheProviders.ToDictionary(c => c.Name, c => c);
        }

        public Dictionary<string, ICacheProvider> CacheProviders { get; }

        public async Task<object> Get(string type, string cacheKey)
        {
            if (!CacheProviders.ContainsKey(type))
            {
                throw new DbRESTFulException($"Unsupported cache type: {type}.");
            }

            return await CacheProviders[type].Get(cacheKey);
        }

        public async Task<string> GetString(string cacheKey, string type = "redis")
        {
            if (!CacheProviders.ContainsKey(type))
            {
                throw new DbRESTFulException($"Unsupported cache type: {type}.");
            }

            return await CacheProviders[type].GetString(cacheKey);
        }

        public async Task Set(string type, string cacheKey, object executionResult, int expiration)
        {
            if (!CacheProviders.ContainsKey(type))
            {
                throw new DbRESTFulException($"Unsupported cache type: {type}.");
            }

            await CacheProviders[type].Set(cacheKey, executionResult, expiration);
        }

        public async Task SetString(string cacheKey, string type = "redis")
        {
            await Task.CompletedTask;
            throw new System.NotImplementedException();
        }
    }
}
