﻿using Newtonsoft.Json.Linq;
using System;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Cache
{
    /// <summary>
    /// 缓存服务实现接口
    /// </summary>
    public interface ICacheProvider
    {
        string Name { get; }

        /// <summary>
        /// 获取缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        Task<JToken> Get(string key);

        /// <summary>
        /// 获取字符串类型的缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        Task<string> GetString(string key);

        /// <summary>
        /// 设置缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="data">指定的数据</param>
        /// <param name="expirationInSeconds">指定过期时间（以秒为单位）</param>
        Task Set(string key, object data, int expirationInSeconds);

        /// <summary>
        /// 设置缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="data">指定的数据</param>
        /// <param name="expiresAt">指定过期时间点</param>
        Task Set(string key, object data, DateTime expiresAt);

        /// <summary>
        /// 设置缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="data">指定的数据</param>
        Task Set(string key, object data);

        /// <summary>
        /// 设置缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="data">指定的数据</param>
        /// <param name="expirationInSeconds">指定过期时间（以秒为单位）</param>
        Task SetString(string key, string data, int expirationInSeconds);

        /// <summary>
        /// 设置缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="data">指定的数据</param>
        /// <param name="expiresAt">指定过期时间点</param>
        Task SetString(string key, string data, DateTime expiresAt);

        /// <summary>
        /// 设置缓存值
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="data">指定的数据</param>
        Task SetString(string key, string data);

        /// <summary>
        /// 删除缓存键
        /// </summary>
        /// <param name="key">指定的键</param>
        Task Remove(params string[] key);

        /// <summary>
        /// 自增
        /// </summary>
        /// <param name="key">指定的键</param>
        /// <param name="expiresAt">指定过期时间点</param>
        /// <param name="defaultValue">默认值</param>
        /// <param name="delta">自增步长</param>
        /// <returns></returns>
        Task<ulong> Increase(string key, DateTime expiresAt, ulong defaultValue = 1, ulong delta = 1);

        Task<ulong> GetIncr(string key, bool del = true);

        event EventHandler OnError;
    }
}