﻿using Enyim.Caching;
using Enyim.Caching.Memcached;
using Newtonsoft.Json.Linq;
using System;

namespace Wicture.DbRESTFul.Cache
{
    // TODO: https://github.com/enyim/Memcached2
    //      https://github.com/vieapps/Enyim.Caching
    public class MemCachedProvider : ICacheProvider
    {
        private readonly IMemcachedClient _memcachedClient;

        public MemCachedProvider(IMemcachedClient memcachedClient)
        {
            _memcachedClient = memcachedClient;
        }

        public string Name { get; } = "memcached";

        public event EventHandler OnError;

        public JToken Get(string key)
        {
            try
            {
                var result = _memcachedClient.Get<object>(key);
                if (result == null) return null;
                return (result is JToken) ? (result as JToken) : JToken.FromObject(result);
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"Get from Cache failed for key: `{key}`");
                return null;
            }
        }

        public ulong Increase(string key, DateTime expiresAt, ulong defaultValue = 1, ulong delta = 1)
        {
            try
            {
                return _memcachedClient.Increment(key, defaultValue, delta, expiresAt);
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"Increase failed for key: `{key}`");
                OnError?.Invoke(this, null);
                return 0;
            }
        }

        public ulong GetIncr(string key, bool del = true)
        {
            try
            {
                if (_memcachedClient.TryGet(key, out ulong val))
                {
                    _memcachedClient.Remove(key);
                }
                return val;
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"GetIncr failed for key: `{key}`");
                OnError?.Invoke(this, null);
                return 0;
            }
        }

        public void Remove(string key)
        {
            try
            {
                _memcachedClient.Remove(key);
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"Remove Cache failed for key: `{key}`");
                OnError?.Invoke(this, null);
            }
        }

        public void Set(string key, object data, int expirationInSeconds)
        {
            try
            {
                _memcachedClient.Store(StoreMode.Set, key, data, TimeSpan.FromSeconds(expirationInSeconds));
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"Set Cache failed for key: `{key}`");
                OnError?.Invoke(this, null);
            }
        }

        public void Set(string key, object data, DateTime expiresAt)
        {
            try
            {
                _memcachedClient.Store(StoreMode.Set, key, data, expiresAt);
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"Set Cache failed for key: `{key}`");
                OnError?.Invoke(this, null);
            }
        }

        public void Set(string key, object data)
        {
            try
            {
                _memcachedClient.Store(StoreMode.Set, key, data);
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Error(ex, $"Set Cache failed for key: `{key}`");
                OnError?.Invoke(this, null);
            }
        }

        public IMemcachedClient MemcachedClient { get { return _memcachedClient; } }
    }
}
