﻿using System;
using System.IO;

namespace Wicture.DbRESTFul.Configuration
{
    public static class ConfigurationManager
    {
        private static IConfigurationProvider configurationProvider;
        public static IConfigurationProvider ConfigurationProvider
        {
            get
            {
                return configurationProvider;
            }
            set
            {
                if(configurationProvider != null) configurationProvider.Loaded -= HandleOnLoadedEvent;
                if (configurationProvider != value)
                {
                    configurationProvider = value;
                    configurationProvider.Loaded += HandleOnLoadedEvent;
                }
            }
        }

        private static void HandleOnLoadedEvent(object sender, EventArgs args)
        {
            OnLoaded?.Invoke(sender, args);
        }

        public static Settings Settings { get; set; }

        public static event EventHandler OnLoaded;

        public static void Load(string configFilePath = "appsettings.json")
        {
            if (ConfigurationProvider == null)
            {
                var fullFilePath = configFilePath;
                if (fullFilePath.IndexOf('\\') < 0 && fullFilePath.IndexOf('/') < 0) 
                { 
                    fullFilePath = Path.Combine(Directory.GetCurrentDirectory(), configFilePath);
                }
                ConfigurationProvider = new JsonConfigurationProvider() { ConfigFilePath = fullFilePath };
            }

            try
            {
                Settings = ConfigurationProvider.LoadConfig();
            }
            catch (Exception ex)
            {
                LoggerManager.Logger.Fatal(ex, "Load config file failed.");
                throw ex;
            }
        }
    }
}
