﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Configuration
{
    public class JsonConfigurationProvider : IConfigurationProvider
    {
        public string ConfigFilePath { get; set; }

        public event EventHandler Loaded;

        protected void RaiseOnLoaded()
        {
            Loaded?.Invoke(null, null);
        }

        public virtual Settings LoadConfig()
        {
            if (string.IsNullOrEmpty(ConfigFilePath)) throw new Exception($"The config file path is not specified.");
            if (!File.Exists(ConfigFilePath)) throw new FileNotFoundException($"The config file not exists in path: { ConfigFilePath }.");

            var content = File.ReadAllText(ConfigFilePath);
            var config = JsonConvert.DeserializeObject<JObject>(content);

            // Load override config file. naming convention: config.*.json
            foreach (var filePath in Directory.GetFiles(Path.GetDirectoryName(ConfigFilePath), "appsettings.*.json")?.OrderBy(f => f))
            {
                if (filePath.Equals(ConfigFilePath, StringComparison.InvariantCultureIgnoreCase)) continue;

                LoggerManager.Logger.Info($"Loadding config file: {filePath}.");
                var text = File.ReadAllText(filePath);
                var overrideConfig = JsonConvert.DeserializeObject<JObject>(text);
                config.MergeInto(overrideConfig);
            }

            var settings = config.ToObject<Settings>();
            settings.Document = config;

            var dir = Path.GetDirectoryName(ConfigFilePath);

            RaiseOnLoaded();

            return settings;
        }

        public bool Accessible(HttpContext context)
        {
            LoggerManager.Logger.Info($"Accessible Host: {context.Request.Host.Host} from JsonConfigurationProvider.");
            return context.Request.Host.Host == "localhost";
        }

        public async Task ReloadAsync(HttpContext context)
        {
            await Task.Run(() => ConfigurationManager.Settings = LoadConfig());
        }

        public async Task<object> GetCurrentConfigAsync(HttpContext context)
        {
            return await Task.FromResult(ConfigurationManager.Settings.Document);
        }
    }
}
