﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Wicture.DbRESTFul.Configuration
{
    public class SectionConfigProvider : ISectionConfigProvider
    {
        public static List<ISectionConfig> SectionConfigs { get; private set; } = new List<ISectionConfig>();

        static SectionConfigProvider()
        {
            ConfigurationManager.OnLoaded += HandleOnLoadedEvent;
        }

        private static void HandleOnLoadedEvent(object sender, EventArgs args)
        {
            // Reload the configuration dynamically for section configurations.
            foreach (var config in SectionConfigs)
            {
                config.Load();
                LoggerManager.Logger.Info($"Reload section config for {config.SectionName}.");
            }
        }

        public T Provide<T>() where T : ISectionConfig, new()
        {
            if (SectionConfigs.FirstOrDefault(t => t.GetType() == typeof(T)) is T config)
            {
                return config;
            }

            config = new T();
            config.Load();

            SectionConfigs.Add(config);

            return config;
        }
    }
}
