﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Infrastructure.Executors
{
    public class ApiExecutorProxy : IExecutorProxy
    {
        private IEnumerable<IExecutor> _executors;

        public ApiExecutorProxy(IEnumerable<IExecutor> executors)
        {
            _executors = executors;
        }

        public async Task<object> ExecuteAsync(ExecutionContext context)
        {
            var executor = _executors.FirstOrDefault(e => e.ImplementationType == context.Api.implementation.type);
            if (executor == null)
            {
                throw new DbRESTFulException($"The implementation type '{ context.Api.implementation.type }' is not supported.");
            }

            return await executor.ExecuteAsync(context);
        }
    }
}
