﻿using Newtonsoft.Json;
using System;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Resources.Api;
using Wicture.DbRESTFul.Rpc.Client;

namespace Wicture.DbRESTFul.Infrastructure.Executors
{
    public class RpcExecutor : IExecutor
    {
        public ImplementationType ImplementationType => ImplementationType.Rpc;

        private readonly IRpcClient _rpcClient;

        public RpcExecutor(IRpcClient rpcClient)
        {
            _rpcClient = rpcClient;
        }

        public async Task<object> ExecuteAsync(ExecutionContext context)
        {
            try
            {
                var result = await _rpcClient.InvokeRpcAsync(context);

                if (result.StatusCode == ResultCode.OK)
                {
                    return JsonConvert.DeserializeObject(result.Data);
                }
                else if(result.LogicalException)
                {
                    throw new LogicalException(result.ErrorMessage, result.StatusCode);
                }
                else
                {
                    throw new DbRESTFulException(result.StatusCode, result.ErrorMessage);
                }
            }
            catch (Exception ex)
            {
                if (ex.InnerException != null && ex.InnerException is LogicalException)
                {
                    throw ex.InnerException;
                }
                else
                {
                    throw ex;
                }
            }
        }
    }
}
