﻿using Autofac.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.HttpOverrides;
using Microsoft.AspNetCore.Routing;
using System;
using System.Linq;
using Wicture.DbRESTFul.Infrastructure;
using Wicture.DbRESTFul.Resources;
using Wicture.DbRESTFul.Resources.Api;
using Wicture.DbRESTFul.Rpc;

namespace Wicture.DbRESTFul
{
    public static class DbRESTFulApplicationBuilderExtensions
    {
        public static IApplicationBuilder UseDbRESTFul(this IApplicationBuilder app, Action<DbRESTFulOptions> setupAction = null)
        {
            app.UseForwardedHeaders(new ForwardedHeadersOptions { ForwardedHeaders = ForwardedHeaders.XForwardedFor | ForwardedHeaders.XForwardedProto });

            AutofacContainer.Container = app.ApplicationServices.GetAutofacRoot();

            var resourceLoader = app.ApplicationServices.GetService(typeof(IResourceManager)) as IResourceManager;
            resourceLoader.LoadAll();

            app.UseMvc(routes => BuildRouteForControllerImplementation(routes, resourceLoader));
            app.UseMiddleware<DbRESTFulApiMiddleware>();

            var options = new DbRESTFulOptions();
            setupAction?.Invoke(options);
            if (options.EnableRpc) app.UseRpc();

            return app;
        }

        private static void BuildRouteForControllerImplementation(IRouteBuilder routes, IResourceManager resourceLoader)
        {
            foreach (var api in resourceLoader.Apis.Where(a => a.Value.implementation.type == ImplementationType.Controller))
            {
                var parts = api.Value.implementation.name.Split('.', System.StringSplitOptions.RemoveEmptyEntries);
                if (parts.Length == 2)
                {
                    var controller = parts[0].EndsWith("Controller") ? parts[0].Substring(0, parts[0].Length - 10) : parts[0];
                    var action = parts[1];
                    routes.MapRoute(api.Value.name, api.Key, defaults: new { controller, action });
                }
            }
        }
    }
}
