﻿using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using NLog;
using NLog.Web;
using System.IO;
using Wicture.DbRESTFul.Log.NLog;

namespace Wicture.DbRESTFul
{
    public static class LogServiceExtension
    {
        public static IHostBuilder UseNLogService(this IHostBuilder builder, INLogConfigProvider provider = null, NLogAspNetCoreOptions options = null, string nlogConfigFile = "nlog.config")
        {
            builder.UseNLog(options);
            builder.ConfigureServices(services => services.AddLogService(provider, nlogConfigFile));
            return builder;
        }

        private static IServiceCollection AddLogService(this IServiceCollection services, INLogConfigProvider provider, string nlogConfigFile)
        {
            if (!string.IsNullOrEmpty(nlogConfigFile)
                && File.Exists(Path.Combine(Directory.GetCurrentDirectory(), nlogConfigFile)))
            {
                LogManager.Setup().LoadConfigurationFromFile(nlogConfigFile);
            }
            else
            {
                LogManager.Configuration = (provider ?? new DefaultNLogConfigProvider()).Provide();
            }

            return services;
        }
    }
}
