﻿using Newtonsoft.Json;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Authentication;

namespace Wicture.DbRESTFul.Infrastructure.Filters.AuthorizeFilter
{
    public class DefaultAuthorizeFilter : IAuthorizeFilter
    {
        private readonly IUserIdentifier _userIdentifier;
        private readonly IApiAuthenticationService _apiAuthenticationService;

        public DefaultAuthorizeFilter(IApiAuthenticationService apiAuthenticationService,
            IUserIdentifier userIdentifier)
        {
            _apiAuthenticationService = apiAuthenticationService;
            _userIdentifier = userIdentifier;
        }

        public async Task OnAuthorize(ExecutionContext context)
        {
            if (context is HttpApiExecutionContext httpApiExecutionContext)
            {
                context.Identity = await _apiAuthenticationService.Authenticate(httpApiExecutionContext.HttpContext, context.Api);
            }
            else if (context is RpcApiExecutionContext rpcApiExecutionContext)
            {
                if (context.Identity == null)
                {
                    context.Identity = !string.IsNullOrEmpty(rpcApiExecutionContext.Request.Identity)
                        ? JsonConvert.DeserializeObject<UserIdentity>(rpcApiExecutionContext.Request.Identity)
                        : await _apiAuthenticationService.Authenticate(rpcApiExecutionContext.Request.Token, context.Api, "");
                }
            }

            await _userIdentifier.AuthorizeAsync(context);
        }
    }
}
