﻿using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Configuration;
using Wicture.DbRESTFul.Resources.Csi;

namespace Wicture.DbRESTFul.Infrastructure.Filters
{
    public class CsiLogFilter : ICsiFilter
    {
        private readonly ILogger _logger;
        private readonly CSILogConfig _config;

        public CsiLogFilter(ILoggerFactory loggerFactory, 
            ISectionConfigProvider sectionConfigProvider)
        {
            _logger = loggerFactory.CreateLogger("DbRESTFulApi:Csi");
            _config = sectionConfigProvider.Provide<CsiSectionConfig>().LogConfig;
        }

        public async Task OnInvokingAsync(InvokeContext context)
        {
            var entry = new JObject();
            if (_config.Code) entry["Csi"] = JToken.FromObject(context.Csi);
            if (_config.Param) entry["Param"] = JToken.FromObject(context.Param);

            if (entry.Count > 0)
            {
                _logger.LogInformation(entry.ToString());
            }

            await Task.CompletedTask;
        }

        public async Task OnInvokedAsync(InvokeContext context)
        {
            if (_config.Result)
            {
                var entry = new JObject();
                entry["Result"] = JToken.FromObject(context.Result);
                _logger.LogInformation(entry.ToString());
            }

            await Task.CompletedTask;
        }
    }
}
