﻿using System.Collections.Generic;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure.Repository.Middlewares;

namespace Wicture.DbRESTFul.Infrastructure.Filters
{
    public class FilterInvoker : IFilterInvoker
    {
        private readonly IResultFilter _resultFilter;
        private readonly IEnumerable<IApiFilter> _apiFilters;
        private readonly IEnumerable<ICsiFilter> _csiFilters;
        private readonly IEnumerable<IExceptionFilter> _excpetionFilters;
        private readonly IMockFilter _mockFilter;
        private readonly IAuthorizeFilter _authorizeFilter;
        private readonly CsiMiddlewareFactory _csiMiddlewareFactory;

        public FilterInvoker(
            IResultFilter resultFilter,
            IEnumerable<IApiFilter> apiFilters,
            IEnumerable<ICsiFilter> csiFilters,
            IEnumerable<IExceptionFilter> excpetionFilters,
            IMockFilter mockFilter,
            IAuthorizeFilter authorizeFilter,
            CsiMiddlewareFactory csiMiddlewareFactory)
        {
            _resultFilter = resultFilter;
            _apiFilters = apiFilters;
            _csiFilters = csiFilters;
            _excpetionFilters = excpetionFilters;
            _mockFilter = mockFilter;
            _authorizeFilter = authorizeFilter;
            _csiMiddlewareFactory = csiMiddlewareFactory;
        }

        public async Task InvokeAuthorizeFilterAsync(ExecutionContext executionContext)
        {
            await _authorizeFilter.OnAuthorize(executionContext);
        }

        public async Task InvokeApiExecutingFiltersAsync(ExecutionContext executionContext)
        {
            foreach (var filter in _apiFilters)
            {
                await filter.OnExecutingAsync(executionContext);
            }
        }

        public async Task InvokeApiExecutedFiltersAsync(ExecutionContext executionContext)
        {
            foreach (var filter in _apiFilters)
            {
                await filter.OnExecutedAsync(executionContext);
            }
        }

        public async Task InvokeCsiInvokingFiltersAsync(InvokeContext invokeContext)
        {
            foreach (var filter in _csiFilters)
            {
                await filter.OnInvokingAsync(invokeContext);
            }
        }

        public async Task InvokeCsiInvokedFiltersAsync(InvokeContext invokeContext)
        {
            foreach (var filter in _csiFilters)
            {
                await filter.OnInvokedAsync(invokeContext);
            }
        }

        public async Task InvokeApiExceptionFiltersAsync(ExecutionContext executionContext)
        {
            foreach (var filter in _excpetionFilters)
            {
                await filter.OnException(executionContext);
            }
        }

        public async Task InvokeApiMockFilterAsync(ExecutionContext executionContext)
        {
            await _mockFilter.OnMock(executionContext);
        }

        public async Task InvokeApiResultFilterAsync(ExecutionContext executionContext)
        {
            if (executionContext is HttpApiExecutionContext httpApiExecutionContext)
            {
                if (!httpApiExecutionContext.HttpContext.Response.HasStarted)
                {
                    await _resultFilter.OnHttpApiResultExecuting(httpApiExecutionContext);
                    await _resultFilter.OnHttpApiResultComposition(httpApiExecutionContext);
                    await _resultFilter.OnHttpApiResultExecuted(httpApiExecutionContext);
                }
            }
            else if (executionContext is RpcApiExecutionContext rpcApiExecutionContext)
            {
                await _resultFilter.OnRpcApiResultExecuting(rpcApiExecutionContext);
                await _resultFilter.OnRpcApiResultComposition(rpcApiExecutionContext);
                await _resultFilter.OnRpcApiResultExecuted(rpcApiExecutionContext);
            }
        }

        public async Task ProcessCsiMiddlewareAsync(InvokeContext context, ActivePosition position)
        {
            await _csiMiddlewareFactory.Process(context, position);
        }
    }
}
