﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Resources.Api;

namespace Wicture.DbRESTFul.Infrastructure.Filters
{
    public class DefaultMockFilter : IMockFilter
    {
        public async Task OnMock(ExecutionContext context)
        {
            var item = context.Api.mock.FirstOrDefault(mock => Match(mock, context.Param));
            if (item == null) throw new DbRESTFulException($"Unexpected parameters for api `{context.Api.name}`, no mock data matches.");

            context.ComposedResult = item.output?.ToObject<StandardResult>();
            await Task.CompletedTask;
        }

        private bool Match(ApiMock item, JToken param)
        {
            var matched = true;
            foreach (var token in item.input)
            {
                if (token.Value.Type == JTokenType.Array || token.Value.Type == JTokenType.Object)
                {
                    var inputVal = token.Value?.ToString(Formatting.None);
                    var paramVal = param[token.Key]?.ToString(Formatting.None);
                    if (inputVal != paramVal)
                    {
                        matched = false;
                        break;
                    }
                }
                else if (token.Value.Value<string>() != "*" && token.Value.Value<string>() != param[token.Key].Value<string>())
                {
                    matched = false;
                    break;
                }
            }

            return matched;
        }
    }
}
