﻿using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Infrastructure.Filters
{
    public interface IResultFilter
    {
        /// <summary>
        /// Provides HttpApi execution hook posibility before result composition.
        /// </summary>
        Task OnHttpApiResultExecuting(HttpApiExecutionContext context);

        /// <summary>
        /// Provides RpcApi execution hook posibility before result composition.
        /// </summary>
        Task OnRpcApiResultExecuting(RpcApiExecutionContext context);

        /// <summary>
        /// Composes the ExecutionContext to the ComposedResult for writing to client.
        /// </summary>
        Task OnHttpApiResultComposition(HttpApiExecutionContext context);

        /// <summary>
        /// Composes the ExecutionContext to the ComposedResult for writing to client.
        /// </summary>
        Task OnRpcApiResultComposition(RpcApiExecutionContext context);

        /// <summary>
        /// Provides HttpApi execution hook posibility before result composition.
        /// </summary>
        Task OnHttpApiResultExecuted(HttpApiExecutionContext context);

        /// <summary>
        /// Provides RpcApi execution hook posibility before result composition.
        /// </summary>
        Task OnRpcApiResultExecuted(RpcApiExecutionContext context);
    }
}
