﻿using Microsoft.Extensions.Logging;
using System;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Resources;
using Wicture.DbRESTFul.Resources.Api;

namespace Wicture.DbRESTFul.Infrastructure
{
    public class HttpApiRequestHandler : IRequestHandler
    {
        private readonly ILogger _logger;
        private readonly HttpApiRequestProcessor _requestProcessor;
        private readonly IResourceManager _resourceManager;

        public HttpApiRequestHandler(ILoggerFactory loggerFactory,
            HttpApiRequestProcessor requestProcessor,
            IResourceManager resourceManager)
        {
            _logger = loggerFactory.CreateLogger("DbRESTFulApi");
            _requestProcessor = requestProcessor;
            _resourceManager = resourceManager;
        }

        private LogicalException TryGetLogicalException(Exception ex)
        {
            if (ex == null) return null;
            if (ex is LogicalException) return ex as LogicalException;
            return TryGetLogicalException(ex.InnerException);
        }

        public int Order { get; set; } = 10;

        public async Task<bool> Handle(HttpApiExecutionContext execContext)
        {
            var context = execContext.HttpContext;
            var abPath = context.Request.Path.Value;
            if (!TryMapRoute(abPath, context.Request.Method, out ApiModel api))
            {
                _logger.LogDebug($"No API is found for request `{abPath}`.");
                return false;
            }

            if (api.implementation.type == ImplementationType.Controller)
            {
                _logger.LogInformation($"API is routed to the `{api.implementation.name}`.");
                return false;
            }

            execContext.Api = api;
            
            await _requestProcessor.Process(execContext);

            return true;
        }

        private bool TryMapRoute(string path, string method, out ApiModel api)
        {
            var key = (path + "_" + method).Trim('/').ToLowerInvariant();

            bool matched = _resourceManager.Apis.ContainsKey(key);
            api = matched ? _resourceManager.Apis[key] : null;

            return matched;
        }
    }
}
