﻿using Microsoft.Extensions.Logging;
using System;
using System.Net;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure.Executors;
using Wicture.DbRESTFul.Infrastructure.Filters;
using Wicture.DbRESTFul.Infrastructure.Handlers;
using Wicture.DbRESTFul.Cache;
using Wicture.DbRESTFul.Resources.Api;

namespace Wicture.DbRESTFul.Infrastructure
{
    public class HttpApiRequestProcessor : ApiRquestProcessorBase<HttpApiExecutionContext>
    {
        private readonly IRequestDataParser _requestDataParser;

        public HttpApiRequestProcessor(ILoggerFactory loggerFactory,
            IFilterInvoker filterInvoker,
            ICacheService cacheService,
            IExecutorProxy executorProxy,
            IRequestDataParser requestDataParser)
            :base(loggerFactory, filterInvoker, cacheService, executorProxy)
        {
            _requestDataParser = requestDataParser;
        }

        protected override async Task OnRequestVerificationAsync(HttpApiExecutionContext context)
        {
            if (context.Api.protocol == Protocol.RpcOnly)
            {
                var message = $"The protocol HTTP is not supported for api `{context.Api.name}`.";
                throw new DbRESTFulException((int)HttpStatusCode.MethodNotAllowed, message);
            }

            if (!context.HttpContext.Request.Method.Equals(context.Api.method.ToString(), StringComparison.CurrentCultureIgnoreCase))
            {
                var message = $"HTTP Method '{context.HttpContext.Request.Method}' is not supported for request `{context.HttpContext.Request.Path}`. expect: '{context.Api.method}'";
                throw new DbRESTFulException((int)HttpStatusCode.MethodNotAllowed, message);
            }

            await base.OnRequestVerificationAsync(context);
        }

        protected override async Task OnParameterProcessAsync(HttpApiExecutionContext context)
        {
            context.Param = await _requestDataParser.ParseAysnc(context.Api.parameter, context);
        }
    }
}
