﻿using Microsoft.AspNetCore.Http;
using Newtonsoft.Json;
using System;
using System.Net;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Configuration;

namespace Wicture.DbRESTFul.Infrastructure
{
    public class ConfigurationReqeustHandler : IRequestHandler
    {
        public int Order { get; set; } = 30;

        public async Task<bool> Handle(HttpApiExecutionContext execContext)
        {
            var context = execContext.HttpContext;
            if (!context.Request.Path.Value.Equals("/config/settings")) return false;

            if (!ConfigurationManager.ConfigurationProvider.Accessible(context))
            {
                context.Response.StatusCode = (int)HttpStatusCode.Forbidden;
                return true;
            }

            if (context.Request.Method.Equals("GET", StringComparison.InvariantCultureIgnoreCase))
            {
                context.Response.Headers["Content-Type"] = "application/json";
                var config = await ConfigurationManager.ConfigurationProvider.GetCurrentConfigAsync(context);
                await context.Response.WriteAsync(JsonConvert.SerializeObject(config, Formatting.Indented));

            }
            else if (context.Request.Method.Equals("POST", StringComparison.InvariantCultureIgnoreCase))
            {
                // Updated config on the fly: Reload.
                await ConfigurationManager.ConfigurationProvider.ReloadAsync(context);
                LoggerManager.Logger.Info($"Reload configuration from {ConfigurationManager.ConfigurationProvider.GetType().Name}.");
            }
            else
            {
                context.Response.StatusCode = (int)HttpStatusCode.Forbidden;
            }

            return true;
        }
    }
}
