﻿using Microsoft.Extensions.Logging;
using Newtonsoft.Json.Linq;
using System.Net;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Configuration;
using Wicture.DbRESTFul.Infrastructure.Executors;
using Wicture.DbRESTFul.Infrastructure.Filters;
using Wicture.DbRESTFul.Cache;
using Wicture.DbRESTFul.Resources.Api;
using Wicture.DbRESTFul.Rpc;

namespace Wicture.DbRESTFul.Infrastructure
{
    public class RpcApiRequestProcessor : ApiRquestProcessorBase<RpcApiExecutionContext>
    {
        private readonly RpcSectionConfig _rpcSectionConfig;
      
        public RpcApiRequestProcessor(ILoggerFactory loggerFactory,
            IFilterInvoker filterInvoker,
            ICacheService cacheService,
            ISectionConfigProvider sectionConfigProvider,
            IExecutorProxy executorProxy)
            : base(loggerFactory, filterInvoker, cacheService, executorProxy)
        {
            _rpcSectionConfig = sectionConfigProvider.Provide<RpcSectionConfig>();
        }

        protected override async Task OnRequestVerificationAsync(RpcApiExecutionContext context)
        {
            if (_rpcSectionConfig.Server.ServiceName != context.Request.ServiceName)
            {
                throw new DbRESTFulException(ResultCode.InternalError, $"The service name does not match, expected `{_rpcSectionConfig.Server.ServiceName}`");
            }

            if (context.Api == null)
            {
                throw new DbRESTFulException(ResultCode.InternalError, $"No api {context.Request.ApiName} found in service `{_rpcSectionConfig.Server.ServiceName}`");
            }

            if (context.Api.protocol == Protocol.HttpOnly)
            {
                var message = $"The protocol Rpc is not supported for api `{context.Api.name}`.";
                throw new DbRESTFulException((int)HttpStatusCode.MethodNotAllowed, message);
            }

            await base.OnRequestVerificationAsync(context);
        }

        protected override async Task OnParameterProcessAsync(RpcApiExecutionContext context)
        {
            context.Param = JToken.Parse(context.Request.Params);
            await Task.CompletedTask;
        }
    }
}
