﻿using Autofac;
using System.Reflection;
using Wicture.DbRESTFul.Cache;
using Wicture.DbRESTFul.Infrastructure.Filters;
using Wicture.DbRESTFul.Infrastructure.IoC;
using Wicture.DbRESTFul.Infrastructure.Repository;
using Wicture.DbRESTFul.Resources;

namespace Wicture.DbRESTFul
{
    public static class AutofacContainer
    {
        public static ILifetimeScope Container { get; set; }

        public static void RegisterDbRESTFul(this ContainerBuilder containerBuilder, params Assembly[] assemblies)
        {
            containerBuilder.RegisterAssemblyTypes(typeof(AutofacContainer).Assembly).AsImplementedInterfaces();

            containerBuilder.RegisterModule<DbRESTFulCoreModule>();

            if (assemblies.Length == 0)
            {
                assemblies = new Assembly[] { Assembly.GetCallingAssembly() };
            }

            var dbrType = typeof(DbRESTFulRepository);

            foreach (var assembly in assemblies)
            {
                containerBuilder.RegisterAssemblyTypes(assembly).AsImplementedInterfaces();
                containerBuilder.RegisterAssemblyTypes(assembly).OnActivating(e =>
                {
                    if (e.Instance is DbRESTFulRepository repository)
                    {
                        repository.ConnectionManager = e.Context.Resolve<IConnectionManager>();
                        repository.FilterInvoker = e.Context.Resolve<IFilterInvoker>();
                        repository.ResourceManager = e.Context.Resolve<IResourceManager>();
                    }
                }).AsSelf();
            }
        }
    }
}
