﻿using Autofac;
using Wicture.DbRESTFul.Authentication;
using Wicture.DbRESTFul.Cache;
using Wicture.DbRESTFul.Configuration;
using Wicture.DbRESTFul.Infrastructure.Filters;
using Wicture.DbRESTFul.Infrastructure.Handlers;
using Wicture.DbRESTFul.Infrastructure.Repository;
using Wicture.DbRESTFul.Infrastructure.Repository.Middlewares;
using Wicture.DbRESTFul.Resources;
using Wicture.DbRESTFul.Resources.Api;
using Wicture.DbRESTFul.Resources.Csi;
using Wicture.DbRESTFul.Resources.Repository;
using Wicture.DbRESTFul.Rpc.Server;

namespace Wicture.DbRESTFul.Infrastructure.IoC
{
    public class DbRESTFulCoreModule : Module
    {
        protected override void Load(ContainerBuilder builder)
        {
            builder.RegisterType<RedisCacheProvider>().SingleInstance();
            builder.RegisterType<MemoryCacheProvider>().SingleInstance();

            builder.RegisterType<DefaultCacheService>().As<ICacheService>().SingleInstance();

            builder.RegisterType<SqlConnectionManager>().As<IConnectionManager>().SingleInstance();
            builder.RegisterType<FilterInvoker>().As<IFilterInvoker>().SingleInstance();
            builder.RegisterType<DefaultTokenOptionsProvider>().As<ITokenOptionsProvider>().SingleInstance();
            builder.RegisterType<ExclusiveIdentityStateManager>().As<IIdentityStateManager>().SingleInstance();
            builder.RegisterType<SimpleDbConnectionRouter>().As<IDbConnectionRouter>().SingleInstance();
            builder.RegisterType<SectionConfigProvider>().As<ISectionConfigProvider>().SingleInstance();

            builder.RegisterType<HttpApiRequestProcessor>().AsSelf().SingleInstance();
            builder.RegisterType<RpcApiRequestProcessor>().AsSelf().SingleInstance();
            builder.RegisterType<DbRESTFulRpcService>().AsSelf().SingleInstance();
            builder.RegisterType<DbRESTFulRpcServer>().AsSelf().SingleInstance();

            builder.RegisterType<RequestDataParser>().As<IRequestDataParser>().SingleInstance();

            builder.RegisterType<ApiLoader>().AsSelf().SingleInstance();
            builder.RegisterType<CsiLoader>().AsSelf().SingleInstance();
            builder.RegisterType<RepositoryLoader>().AsSelf().SingleInstance();

            builder.RegisterType<CsiMiddlewareFactory>().AsSelf().SingleInstance();

            builder.RegisterType<ResourceManager>().As<IResourceManager>().SingleInstance();

            builder.RegisterType<DbRESTFulRepository>()
                .OnActivating(e =>
                {
                    e.Instance.ConnectionManager = e.Context.Resolve<IConnectionManager>();
                    e.Instance.FilterInvoker = e.Context.Resolve<IFilterInvoker>();
                    e.Instance.ResourceManager = e.Context.Resolve<IResourceManager>();
                })
                .AsSelf().As<IDbRESTFulRepository>().InstancePerLifetimeScope();     
            
            base.Load(builder);
        }
    }
}
