﻿using MySql.Data.MySqlClient;
using Npgsql;
using System;
using System.Data;
using System.Data.SqlClient;
using Wicture.DbRESTFul.Resources.Csi;

namespace Wicture.DbRESTFul.Infrastructure.Repository
{
    public class SqlConnectionManager : IConnectionManager
    {
        private IDbConnectionRouter _dbConnectionRouter;

        public DbConnection DbConnection { get; set; }

        public SqlConnectionManager(IDbConnectionRouter dbConnectionResolver)
        {
            _dbConnectionRouter = dbConnectionResolver;
        }

        public void Resolve(ExecutionContext executionContext)
        {
            DbConnection = _dbConnectionRouter.Resolve(executionContext);
        }

        public IDbConnection GetConnection(bool queryOnly = true)
        {
            // Try default DbConnection if not resolved.
            DbConnection = DbConnection ?? _dbConnectionRouter.DefaultDbConnection;

            var connectionString = queryOnly ? DbConnection.ReadConnectionString : DbConnection.WriteConnectionString;

            switch (DbConnection.DatabaseType)
            {
                case DatabaseType.MySQL:
                    return new MySqlConnection(connectionString);
                case DatabaseType.SQLServer:
                    return new SqlConnection(connectionString);
                case DatabaseType.Postgres:
                    return new NpgsqlConnection(connectionString);
                default:
                    throw new NotSupportedException($"Databasetype `{DbConnection.DatabaseType}` is not supported.");
            }
        }
    }
}