﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Infrastructure.Repository.Middlewares
{
    public class CsiMiddlewareFactory
    {
        private readonly Dictionary<string, ICsiMiddleware> _csiMiddlewares;

        public CsiMiddlewareFactory(IEnumerable<ICsiMiddleware> csiMiddlewares)
        {
            _csiMiddlewares = csiMiddlewares.ToDictionary(m => m.Name, m => m);
        }

        public async Task Process(InvokeContext context, ActivePosition position)
        {
            if (context.Csi.middlewares == null) return;

            try
            {
                foreach (var item in context.Csi.middlewares)
                {
                    if (_csiMiddlewares.ContainsKey(item.Key))
                    {
                        var middleWare = _csiMiddlewares[item.Key];
                        if (middleWare.ActivePosition == position)
                        {
                            var message = middleWare.Validate(context, item.Value);
                            if (string.IsNullOrEmpty(message))
                            {
                                await middleWare.ProcessAsync(context, item.Value);
                            }
                            else
                            {
                                throw new DbRESTFulException($"Resolving middleware '{item.Key}' failed: {message}");
                            }
                        }
                    }
                    else
                    {
                        throw new DbRESTFulException($"Unknown middleware '{item.Key}'.");
                    }
                }
            }
            catch (Exception ex)
            {
                throw new DbRESTFulException($"Middleware process error: {ex}");
            }
        }
    }
}