﻿using System.Collections.Generic;
using System.Data;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Cache;
using Wicture.DbRESTFul.Infrastructure.Filters;
using Wicture.DbRESTFul.Resources;

namespace Wicture.DbRESTFul.Infrastructure.Repository
{
    public interface IDbRESTFulRepository
    {
        IFilterInvoker FilterInvoker { get; set; }

        ExecutionContext Context { get; set; }

        IConnectionManager ConnectionManager { get; set; }

        IResourceManager ResourceManager { get; set; }

        /// <summary>
        /// Generic Invoke the csi with parameters, without managing connection and transaction.
        /// </summary>
        Task<dynamic> InvokeAsync(string csiName, object parameters = null, IDbConnection connection = null, IDbTransaction transaction = null);



        /// <summary>
        /// Invoke specified csi to return an IEnumerable<T> type, with ability to share connection and manage transaction
        /// </summary>
        Task<IEnumerable<T>> InvokeAsync<T>(string csiName, object parameters = null, IDbConnection connection = null, IDbTransaction transaction = null);

        /// <summary>
        /// Invoke specified csi to return an single or default T type, with ability to share connection and manage transaction
        /// </summary>
        Task<T> InvokeSingleOrDefaultAsync<T>(string csiName, object parameters = null, IDbConnection connection = null, IDbTransaction transaction = null);


               
        /// <summary>
        /// Query specified sql to return an IEnumerable<T> type, with ability to share connection and manage transaction
        /// </summary>
        Task<IEnumerable<T>> QueryAsync<T>(string sql, object parameters = null, IDbConnection connection = null, IDbTransaction transaction = null, bool queryOnly = true);

        /// <summary>
        /// Query specified sql to return an single or default T type, with ability to share connection and manage transaction
        /// </summary>
        Task<T> QuerySingleOrDefaultAsync<T>(string sql, object parameters = null, IDbConnection connection = null, IDbTransaction transaction = null, bool queryOnly = true);
    }
}
