﻿using Newtonsoft.Json.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Infrastructure.Repository.Middlewares
{
    /// <summary>
    /// 用户身份中间件，CSI中如果需要用到当前用户的userId，userName等作为参数，而调用者无法传入的时候，可以通过该中间件处理。
    /// </summary>
    public class IdentityMiddleware : ICsiMiddleware
    {
        public ActivePosition ActivePosition { get { return ActivePosition.BeforeBuildCommandParam; } }
        public string Name { get { return "identity"; } }

        public string Validate(InvokeContext context, JToken config)
        {
            var message = string.Empty;
            if (!(config is JObject))
            {
                message = "config type should be an object.";
            }
            
            return message;
        }

        /// <summary>
        /// Parse the middle ware to parameters.
        /// </summary>
        /// <param name="config">
        /// {
        ///     "userId": "operator",
        ///     "userName": "userName",
        ///     "alias": "alias",
        ///     "role": "role"
        /// }
        /// </param>
        public async Task ProcessAsync(InvokeContext context, JToken config)
        {
            if (context.Param is JArray)
            {
                foreach (var param in context.Param as JArray)
                {
                    SetIdentity(context, config, param);
                }
            }
            else
            {
                var param = context.Param as JObject;
                SetIdentity(context, config, param);
            }

            await Task.CompletedTask;
        }

        private static void SetIdentity(InvokeContext context, JToken config, JToken param)
        {
            var identity = config as JObject;

            var key_uid = identity.Value<string>("userId");
            if (!string.IsNullOrEmpty(key_uid) && !string.IsNullOrEmpty(context.Repository.Context.Identity.Id))
            {
                param[key_uid] = context.Repository.Context.Identity.Id;
            }

            var key_un = identity.Value<string>("userName");
            if (!string.IsNullOrEmpty(key_un) && !string.IsNullOrEmpty(context.Repository.Context.Identity.Name))
            {
                param[key_un] = context.Repository.Context.Identity.Name;
            }

            var key_ur = identity.Value<string>("role");
            if (!string.IsNullOrEmpty(key_ur) && !string.IsNullOrEmpty(context.Repository.Context.Identity.Role))
            {
                param[key_ur] = context.Repository.Context.Identity.Role;
            }

            var key_ua = identity.Value<string>("alias");
            if (!string.IsNullOrEmpty(key_ua) && !string.IsNullOrEmpty(context.Repository.Context.Identity.Alias))
            {
                param[key_ua] = context.Repository.Context.Identity.Alias;
            }
        }
    }
}