﻿using Newtonsoft.Json.Linq;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Infrastructure.Repository.Middlewares
{
    /// <summary>
    /// 参数预处理中间件，将当前用户输入的参数通过指定的一系列方法的处理，最终合并回到参数列表中。
    /// 此中间件可以解决需要对参数作一些处理，加工的使用场景，比如：用户注册，需要将密码字符串加密等。
    /// </summary>
    public class PreprocessMiddleware : ICsiMiddleware
    {
        private readonly Dictionary<string, IParameterPreprocessor> _preprocessors;

        public PreprocessMiddleware(IEnumerable<IParameterPreprocessor> parameterPreprocessors)
        {
            _preprocessors = parameterPreprocessors.ToDictionary(p => p.GetType().Name, p => p);
        }

        public ActivePosition ActivePosition { get { return ActivePosition.BeforeBuildCommandParam; } }

        public string Name { get { return "preprocess"; } }

        public string Validate(InvokeContext context, JToken config)
        {
            var message = string.Empty;

            if (config.Type != JTokenType.Array)
            {
                message = "The preprocess config should be an Array of the Preprocessor names.";
            }

            return message;
        }

        public async Task ProcessAsync(InvokeContext context, JToken config)
        {
            foreach (var item in (config as JArray))
            {
                var processorName = item.Value<string>();
                if (!_preprocessors.ContainsKey(processorName))
                {
                    throw new DbRESTFulException("The preprocessor `{processorName}` is not found.");
                }

                var processor = _preprocessors[processorName];
                await processor.Process(context);
            }
        }
    }
}