﻿using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace Wicture.DbRESTFul.Infrastructure.Repository.Middlewares
{
    public class ArrayParameterPreprocessor : IParameterPreprocessor
    {
        public async Task Process(InvokeContext context)
        {
            if (context.Param.Type == JTokenType.Object)
            {
                var obj = context.Param as JObject;
                var keys = (obj as IDictionary<string, JToken>).Keys.ToArray();
                foreach (var key in keys)
                {
                    if (key.EndsWith("_Array", StringComparison.InvariantCultureIgnoreCase) && obj[key].Type == JTokenType.String)
                    {
                        var newKey = key.Substring(0, key.Length - 6);
                        var items = obj.Value<string>(key).Split("".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);
                        context.Param[newKey] = new JArray(items);
                        context.Param.RemoveProperties(key);
                    }
                }
            }

            await Task.CompletedTask;
        }
    }
}
