﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using Wicture.DbRESTFul.Configuration;

namespace Wicture.DbRESTFul.Resources.Api
{
    public class ApiLoader : IResourceLoader<ApiModel>
    {
        private const string APIProfix = "/api/";
        private const string apiFileExtention = "*.*";

        public Dictionary<string, ApiModel> Items { get; } = new Dictionary<string, ApiModel>();

        public event EventHandler Loaded;

        private readonly ApiSectionConfig _apiConfig;

        public ApiLoader(ISectionConfigProvider sectionConfigProvider)
        {
            _apiConfig = sectionConfigProvider.Provide<ApiSectionConfig>();
        }

        public void Load()
        {
            Items.Clear();

            var profix = string.IsNullOrEmpty(_apiConfig.Prefix) ? APIProfix : _apiConfig.Prefix;
            if (!profix.StartsWith('/')) profix = "/" + profix;
            if (!profix.EndsWith('/')) profix = profix + "/";

            if (string.IsNullOrEmpty(_apiConfig.Path))
            {
                throw new ResourceLoadingException("API path is not specified.");
            }

            if (!Directory.Exists(_apiConfig.Path))
            {
                throw new ResourceLoadingException("API path not Exists");
            }

            LoadFile(_apiConfig.Path, profix);

            Loaded?.Invoke(this, null);
        }

        private void LoadFile(string dir, string profix)
        {
            string lastFile = string.Empty;
            try
            {
                if (!Directory.Exists(dir)) return;

                foreach (var file in Directory.GetFiles(dir, apiFileExtention))
                {
                    lastFile = file;
                    var text = File.ReadAllText(file);
                    var data = JsonConvert.DeserializeObject<List<ApiModel>>(text);
                    foreach (var item in data)
                    {
                        var url = item.useAbsoluteUrl ? item.url : (profix + item.module + "/" + item.url);
                        var key = (url + "_" + item.method.ToString()).Trim('/').ToLowerInvariant();
                        if (Items.ContainsKey(key))
                            throw new ResourceLoadingException($"The request {key} already exists for API {item.name} in file '{file}'");

                        Items.Add(key, item);
                    }
                }

                foreach (var subDir in Directory.GetDirectories(dir))
                {
                    LoadFile(subDir, profix);
                }
            }
            catch (Exception ex)
            {
                throw new ResourceLoadingException($"Load Api failed, file: {lastFile}", ex);
            }
        }
    }
}
