﻿using Newtonsoft.Json.Linq;
using System;

namespace Wicture.DbRESTFul.Resources.Api
{
    public class ApiModel : IResourceModel
    {
        public string version { get; set; }
        public string owner { get; set; }
        public string name { get; set; }
        public string module { get; set; }
        public string url { get; set; }
        public HttpMethod method { get; set; }
        public string title { get; set; }
        public string summary { get; set; }
        public string note { get; set; }
        public bool allowAnonymous { get; set; } = true;
        public bool useAbsoluteUrl { get; set; } = false;
        public bool deprecated { get; set; }
        public Protocol protocol { get; set; }
        public ApiCache cache { get; set; }
        public bool implemented { get; set; }
        public ApiImplementation implementation { get; set; }
        public ApiParameter parameter { get; set; }
        public ApiResult result { get; set; }
        public ApiMock[] mock { get; set; }
        public DateTime createdAt { get; set; }
    }

    public class Schema
    {
        public string name { get; set; }
        public SchemaType type { get; set; }
        public bool? nullable { get; set; }
        public string description { get; set; }

        /// <summary>
        /// 如果 type 是 SimpleArray 时，可以指定简单类型
        /// </summary>
        public SchemaType? simpleArrayItemType { get; set; }
        public Schema[] schema { get; set; }
    }

    public class ApiParameter
    {
        public Schema[] query { get; set; }
        public BodyType bodyType { get; set; }
        public Schema[] body { get; set; }
    }

    public class ApiImplementation
    {
        public ImplementationType type { get; set; }
        public string name { get; set; }
    }

    public class ApiResult
    {
        public string type { get; set; }
        public Schema[] schema { get; set; }
    }

    public class ApiMock
    {
        public JObject input { get; set; }
        public JToken output { get; set; }
    }

    public class ApiCache
    {
        public bool enabled { get; set; }
        public string type { get; set; }
        public int expiration { get; set; }
    }

    public enum HttpMethod
    {
        GET,
        POST,
        PUT,
        DELETE,
        HEAD,
    }

    public enum ImplementationType
    {
        Csi,
        Repository,
        Controller,
        Rpc
    }

    public enum Protocol
    {
        HttpAndRpc,
        HttpOnly,
        RpcOnly
    }

    public enum BodyType
    {
        Object,
        Array
    }

    public enum SchemaType
    {
        Undefined,
        Object,
        Array,
        SimpleArray, // 简单类型数组 （非Object, Array, SimpleArray）
        String,
        Int,
        Bool,
        Long,
        Float,
        Double,
        Decimal,
        Number,
        Datetime,
        Binary,
        Bit,
        File,
        Boolean
    }
}
