﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Wicture.DbRESTFul.Resources.Csi
{
    public class CsiModel : IResourceModel
    {
        public string name { get; set; }

        public string module { get; set; }

        public JToken code { get; set; }

        public string resultSet { get; set; }

        public bool queryOnly { get; set; }

        public bool requiredTransaction { get; set; }

        public JObject middlewares { get; set; }

        public string branch { get; set; }

        public string description { get; set; }

        public static CsiModel Clone(CsiModel codeItem)
        {
            var result = new CsiModel
            {
                name = codeItem.name,
                requiredTransaction = codeItem.requiredTransaction,
                resultSet = codeItem.resultSet,
                queryOnly = codeItem.queryOnly,
                middlewares = codeItem.middlewares
            };

            result.code = codeItem.code is JValue
                ? new string(codeItem.code.ToString().ToCharArray())
                : JsonConvert.DeserializeObject<JToken>(codeItem.code.ToString());

            return result;
        }

        public static CsiModel Build(string sql, bool singleResult, bool queryOnly = true, bool requiredTransaction = false)
        {
            var result = new CsiModel
            {
                name = "Dynamic",
                requiredTransaction = requiredTransaction,
                resultSet = singleResult ? "S" : "M",
                queryOnly = queryOnly,
                code = sql
            };

            return result;
        }


        public void EnsureSingleResultSet()
        {
            if (!IsSingleResultSet)
            {
                throw new DbRESTFulException("Multi resultSet is not supported, please use InvokeAsync with dynamic result type signature.");
            }
        }

        [JsonIgnore]
        public bool IsSingleResultSet { get { return code.Type == JTokenType.String; } }

        public List<ResultSetInfo> ListResultSetInfo()
        {
            var result = new List<ResultSetInfo>();
            if (code is JObject codeItems)
            {
                var resultSetItems = resultSet?.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                var i = 0;
                foreach (var code in codeItems)
                {
                    var type = resultSetItems?.Skip(i++).Take(1).FirstOrDefault();
                    result.Add(new ResultSetInfo 
                    { 
                        Key = code.Key, 
                        Sql = code.Value.Value<string>(), 
                        MultipleRecords = !(type?.Equals("S", StringComparison.CurrentCultureIgnoreCase) == true)
                    });
                }
            }
            else
            {
                result.Add(new ResultSetInfo
                {
                    Key = string.Empty,
                    Sql = code.Value<string>(),
                    MultipleRecords = !(resultSet?.Equals("S", StringComparison.CurrentCultureIgnoreCase) == true)
                });
            }

            return result;
        }
    }

    public class ResultSetInfo
    {
        public bool MultipleRecords { get; set; }
        public string Key { get; set; }
        public string Sql { get; set; }
    }
}