﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using Wicture.DbRESTFul.Resources.Api;
using Wicture.DbRESTFul.Resources.Csi;
using Wicture.DbRESTFul.Resources.Repository;

namespace Wicture.DbRESTFul.Resources
{
    public class ResourceManager : IResourceManager
    {
        private readonly ApiLoader _apiLoader;
        private readonly CsiLoader _csiLoader;
        private readonly RepositoryLoader _repositoryLoader;
        private readonly ILogger _logger;

        public Dictionary<string, ApiModel> Apis => _apiLoader.Items;

        public Dictionary<string, CsiModel> Csis => _csiLoader.Items;

        public Dictionary<string, RepositoryModel> Repositories => _repositoryLoader.Items;

        public List<IResourceLoader> ResourceLoaders { get; }

        public event EventHandler<ResourceLoadedEventArgs> OnLoaded;

        public ResourceManager(ApiLoader apiLoader, CsiLoader csiLoader, RepositoryLoader repositoryLoader, ILoggerFactory loggerFactory)
        {
            _apiLoader = apiLoader;
            _csiLoader = csiLoader;
            _repositoryLoader = repositoryLoader;
            _logger = loggerFactory.CreateLogger("DbRESTFul");

            ResourceLoaders = new List<IResourceLoader>(){ apiLoader, csiLoader, repositoryLoader };
        }

        public void LoadAll()
        {
            foreach (var item in ResourceLoaders)
            {
                try
                {
                    OnLoaded?.Invoke(this, new ResourceLoadedEventArgs { ResourceType = item.GetType() });
                    item.Load();
                }
                catch (ResourceLoadingException ex)
                {
                    _logger.LogWarning($"Loading `{item.GetType().Name}` failed.", ex);
                }
            }
        }

        public void LoadApi()
        {
            _apiLoader.Load();
            OnLoaded?.Invoke(this, new ResourceLoadedEventArgs { ResourceType = _apiLoader.GetType() });
        }

        public void LoadCsi()
        {
            _csiLoader.Load();
            OnLoaded?.Invoke(this, new ResourceLoadedEventArgs { ResourceType = _csiLoader.GetType() });
        }
    }
}
