﻿using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Configuration;

namespace Wicture.DbRESTFul.Rpc.Client
{
    public class DefaultGrpcChannelProvider : IGrpcChannelProvider
    {
        private readonly ILogger _logger;
        private readonly Dictionary<string, RpcChannel> _channels;

        public DefaultGrpcChannelProvider(ILoggerFactory loggerFactory, ISectionConfigProvider sectionConfigProvider)
        {
            _logger = loggerFactory.CreateLogger<DefaultGrpcChannelProvider>();
            var config = sectionConfigProvider.Provide<RpcSectionConfig>();
            _channels = config.Clients?.ToDictionary(c => c.ServiceName, c => new RpcChannel(c, _logger));
        }

        public async Task<RpcChannel> Provide(string serviceName)
        {
            if (_channels?.Any() != true)
            {
                throw new DbRESTFulException("Rpc client configuration is not set.");
            }

            if (!_channels.ContainsKey(serviceName))
            {
                throw new DbRESTFulException($"ServiceName `{serviceName}` was not found in RpcConfig.");
            }

            var channel = _channels[serviceName];
            return await Task.FromResult(channel);
        }
    }
}
