﻿using Grpc.Core;
using Microsoft.Extensions.Logging;
using System;

namespace Wicture.DbRESTFul.Rpc.Client
{
    public class RpcChannel
    {
        private object locker = new object();
        private Channel channel;
        public RpcClientConfig Config { get; private set; }

        private readonly ILogger _logger;

        public RpcChannel(RpcClientConfig config, ILogger logger)
        {
            Config = config;
            _logger = logger;
        }

        public Channel GetOrOpen()
        {
            if (!Healthy)
            {
                try
                {
                    lock (locker)
                    {
                        if (!Healthy)
                        {
                            channel?.ShutdownAsync().Wait();
                            channel = null;
                        }

                        if (channel == null)
                        {
                            _logger.LogInformation($"New RPC channel was created for service : `{Config.ServiceName}` with `{Config.ServerAddress}`");
                            channel = new Channel(Config.ServerAddress, ChannelCredentials.Insecure);
                        }
                    }
                }
                catch (Exception ex)
                {
                    channel?.ShutdownAsync().Wait();
                    channel = null;
                    _logger.LogError(ex, $"Failed to get available channel for service : `{Config.ServiceName}` with `{Config.ServerAddress}`");
                    throw ex;
                }
            }

            return channel;
        }

        public bool Healthy { get { return !(channel == null || channel.State == ChannelState.Shutdown || channel.State == ChannelState.TransientFailure); } }

        public void Close()
        {
            lock (locker)
            {
                if (channel == null) return;

                try
                {
                    channel.ShutdownAsync().Wait();
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, $"Failed to shutdown channel for service : `{Config.ServiceName}` with `{Config.ServerAddress}`");
                }

                channel = null;
            }
        }
    }
}
