﻿using Grpc.Core;
using Microsoft.Extensions.Logging;
using System;
using Wicture.DbRESTFul.Configuration;

namespace Wicture.DbRESTFul.Rpc.Server
{
    public class DbRESTFulRpcServer
    {
        private readonly DbRESTFulRpcService _rpcService;
        private readonly ILogger _logger;
        private readonly RpcServerConfig _rpcServerConfig;

        public DbRESTFulRpcServer(DbRESTFulRpcService rpcService, ILoggerFactory loggerFactory, ISectionConfigProvider sectionConfigProvider)
        {
            _rpcServerConfig = sectionConfigProvider.Provide<RpcSectionConfig>()?.Server;

            _rpcService = rpcService;
            _logger = loggerFactory.CreateLogger("DbRESTFulRpc");
        }

        public void Start()
        {
            if (_rpcServerConfig?.Enabled != true)
            {
                throw new InvalidOperationException("Rpc Server is not configured or not enabled.");
            }

            var server = new Grpc.Core.Server
            {
                Services = { RpcService.BindService(_rpcService) },
                Ports = { new ServerPort(_rpcServerConfig.Host, _rpcServerConfig.Port, ServerCredentials.Insecure) }
            };

            server.Start();

            _logger.LogInformation($"Rpc server `{_rpcServerConfig.ServiceName}` has been started at `{_rpcServerConfig.Host}:{_rpcServerConfig.Port}`.");
        }
    }
}
