﻿using Grpc.Core;
using Microsoft.Extensions.Logging;
using System;
using System.Linq;
using System.Threading.Tasks;
using Wicture.DbRESTFul.Infrastructure;
using Wicture.DbRESTFul.Resources;
using static Wicture.DbRESTFul.RpcService;

namespace Wicture.DbRESTFul.Rpc.Server
{
    public class DbRESTFulRpcService : RpcServiceBase
    {
        private readonly ILogger _logger;
        private readonly RpcApiRequestProcessor _requestProcessor;
        private readonly IResourceManager _resourceManager;

        public DbRESTFulRpcService(ILoggerFactory loggerFactory, 
            RpcApiRequestProcessor requestProcessor, 
            IResourceManager resourceManager)
        {
            _logger = loggerFactory.CreateLogger("DbRESTFulApiRpc");
            _requestProcessor = requestProcessor;
            _resourceManager = resourceManager;
        }

        public override async Task<RpcResponse> Invoke(RpcRequest request, ServerCallContext context)
        {
            _logger.LogInformation($"RpcService.Invoke api {request.ApiName} in service: {request.ServiceName}.");

            var api = _resourceManager.Apis
                .Values.FirstOrDefault(a => a.name.Equals(request.ApiName, StringComparison.InvariantCultureIgnoreCase));
            var ctx = new RpcApiExecutionContext(request, context) { Api = api };
            await _requestProcessor.Process(ctx);

            return ctx.ComposedResult as RpcResponse;
        }
    }
}
