using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using Wicture.DbRESTFul;

namespace System
{
    public static class Extensions
    {
        public static bool Has(this int value, System.Enum type)
        {
            try
            {
                return ((value & (int)(object)type) == (int)(object)type);
            }
            catch
            {
                return false;
            }
        }

        public static string FormatWith(this string str, params object[] args)
        {
            return string.IsNullOrEmpty(str) ? string.Empty : string.Format(str, args);
        }

        public static bool IsDecimal(this string str)
        {
            return decimal.TryParse(str, out decimal test);
        }

        public static bool IsDecimal(this string str, out decimal test)
        {
            return decimal.TryParse(str, out test);
        }

        public static bool IsDouble(this string str)
        {
            return double.TryParse(str, out double test);
        }

        public static bool IsDouble(this string str, out double test)
        {
            return double.TryParse(str, out test);
        }

        public static string ToFormatString(this decimal str)
        {
            string str1 = str.ToString();
            return str1.IndexOf('.') >= 0 ? str.ToString("#,##0" + ".".PadRight(str1.Substring(str1.IndexOf('.')).Length, '0')) : str.ToString("#,##0");
        }

        public static bool IsInt(this string str)
        {
            return int.TryParse(str, out int test);
        }

        public static bool IsInt(this string str, out int test)
        {
            return int.TryParse(str, out test);
        }

        public static string Join1<T>(this T[] arr, string split = ",")
        {
            StringBuilder sb = new StringBuilder(arr.Length * 36);
            for (int i = 0; i < arr.Length; i++)
            {
                sb.Append(arr[i].ToString());
                if (i < arr.Length - 1)
                {
                    sb.Append(split);
                }
            }
            return sb.ToString();
        }

        public static string RemoveSpace(this string str)
        {
            if (str.IsNullOrEmpty()) return "";
            return str.Replace("", " ").Replace("\r", "").Replace("\n", "");
        }

        public static bool IsLong(this string str)
        {
            return long.TryParse(str, out long test);
        }

        public static bool IsLong(this string str, out long test)
        {
            return long.TryParse(str, out test);
        }

        public static bool IsDateTime(this string str)
        {
            return DateTime.TryParse(str, out DateTime test);
        }

        public static bool IsDateTime(this string str, out DateTime test)
        {
            return DateTime.TryParse(str, out test);
        }

        public static bool IsGuid(this string str)
        {
            return Guid.TryParse(str, out Guid test);
        }

        public static bool IsGuid(this string str, out Guid test)
        {
            return Guid.TryParse(str, out test);
        }

        public static bool IsUrl(this string str)
        {
            if (str.IsNullOrEmpty())
                return false;
            string pattern = @"^(http|https|ftp|rtsp|mms):(\/\/|\\\\)[A-Za-z0-9%\-_@]+\.[A-Za-z0-9%\-_@]+[A-Za-z0-9\.\/=\?%\-&_~`@:\+!;]*$";
            return Regex.IsMatch(str, pattern, RegexOptions.IgnoreCase);
        }

        public static bool IsEmail(this string str)
        {
            return Regex.IsMatch(str, @"^([\w-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([\w-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$");
        }

        public static bool In(this int i, params int[] ints)
        {
            foreach (int k in ints)
            {
                if (i == k)
                {
                    return true;
                }
            }
            return false;
        }

        public static object DBValueOrNull(this string str)
        {
            if (str.IsNullOrEmpty())
            {
                return null;
            }
            else
            {
                return str;
            }
        }

        public static decimal ToDecimal(this string str, int decimals)
        {
            return decimal.TryParse(str, out decimal test) ? Math.Round(test, decimals, MidpointRounding.AwayFromZero) : 0;
        }

        public static decimal ToDecimal(this string str)
        {
            return decimal.TryParse(str, out decimal test) ? test : 0;
        }

        public static decimal Round(this decimal dec, int decimals = 2)
        {
            return Math.Round(dec, decimals, MidpointRounding.AwayFromZero);
        }

        public static double ToDouble(this string str, int digits)
        {
            return double.TryParse(str, out double test) ? test.Round(digits) : 0;
        }

        public static double ToDouble(this string str)
        {
            return double.TryParse(str, out double test) ? test : 0;
        }

        public static double Round(this double value, int decimals)
        {
            if (value < 0)
                return Math.Round(value + 5 / Math.Pow(10, decimals + 1), decimals, MidpointRounding.AwayFromZero);
            else
                return Math.Round(value, decimals, MidpointRounding.AwayFromZero);
        }

        public static short ToShort(this string str)
        {
            short.TryParse(str, out short test);
            return test;
        }

        public static int? ToIntOrNull(this string str)
        {
            if (int.TryParse(str, out int test))
            {
                return test;
            }
            else
            {
                return null;
            }
        }

        public static int ToInt(this object str)
        {
            if (str == null) return 0;

            int.TryParse(str.ToString(), out int test);
            return test;
        }

        public static int ToInt(this string str, int defaultValue)
        {
            return int.TryParse(str, out int test) ? test : defaultValue;
        }

        public static long ToLong(this string str)
        {
            long.TryParse(str, out long test);
            return test;
        }

        public static Int16 ToInt16(this string str)
        {
            Int16.TryParse(str, out short test);
            return test;
        }
        public static Int32 ToInt32(this string str)
        {
            Int32.TryParse(str, out int test);
            return test;
        }

        public static Int64 ToInt64(this string str)
        {
            Int64.TryParse(str, out long test);
            return test;
        }

        public static DateTime ToDateTime(this string str)
        {
            DateTime.TryParse(str, out DateTime test);
            return test;
        }

        public static DateTime? ToDateTimeOrNull(this string str)
        {
            if (DateTime.TryParse(str, out DateTime test))
            {
                return test;
            }
            return null;
        }

        public static Guid ToGuid(this string str)
        {
            if (Guid.TryParse(str, out Guid test))
            {
                return test;
            }
            else
            {
                return Guid.Empty;
            }
        }

        public static bool ToBoolean(this string str)
        {
            return Boolean.TryParse(str, out bool b) ? b : false;
        }

        public static string DateFormat(this object date, string format = "yyyy/MM/dd")
        {
            if (date == null) { return string.Empty; }
            if (!date.ToString().IsDateTime(out DateTime d))
            {
                return date.ToString();
            }
            else
            {
                return d.ToString(format);
            }
        }

        public static bool IsNullOrEmpty(this string str)
        {
            return string.IsNullOrWhiteSpace(str);
        }

        public static string ToString(this IList<string> strList, char split)
        {
            return strList.ToString(split.ToString());
        }

        public static string ToString(this IList<string> strList, string split)
        {
            StringBuilder sb = new StringBuilder(strList.Count * 10);
            for (int i = 0; i < strList.Count; i++)
            {
                sb.Append(strList[i]);
                if (i < strList.Count - 1)
                {
                    sb.Append(split);
                }
            }
            return sb.ToString();
        }

        public static string ReplaceSql(this string str)
        {
            str = str.Replace("'", "").Replace("--", " ").Replace(";", "");
            return str;
        }

        public static string ToChineseSpell(this string strText)
        {
            int len = strText.Length;
            string myStr = "";
            for (int i = 0; i < len; i++)
            {
                myStr += GetSpell(strText.Substring(i, 1));
            }
            return myStr.ToLower();
        }

        public static string GetSpell(this string cnChar)
        {
            byte[] arrCN = Encoding.UTF8.GetBytes(cnChar);
            if (arrCN.Length > 1)
            {
                int area = (short)arrCN[0];
                int pos = (short)arrCN[1];
                int code = (area << 8) + pos;
                int[] areacode = { 45217, 45253, 45761, 46318, 46826, 47010, 47297, 47614, 48119, 48119, 49062, 49324, 49896, 50371, 50614, 50622, 50906, 51387, 51446, 52218, 52698, 52698, 52698, 52980, 53689, 54481 };
                for (int i = 0; i < 26; i++)
                {
                    int max = 55290;
                    if (i != 25) max = areacode[i + 1];
                    if (areacode[i] <= code && code < max)
                    {
                        return Encoding.UTF8.GetString(new byte[] { (byte)(65 + i) });
                    }
                }
                return "x";
            }
            else return cnChar;
        }

        /// <summary>
        /// 截取字符串，汉字两个字节，字母一个字节
        /// </summary>
        /// <param name="str">字符串</param>
        /// <param name="strLength">字符串长度</param>
        /// <returns></returns>
        public static string Interruption(this string str, int len, string show)
        {
            ASCIIEncoding ascii = new ASCIIEncoding();
            int tempLen = 0;
            string tempString = "";
            byte[] s = ascii.GetBytes(str);
            for (int i = 0; i < s.Length; i++)
            {
                if ((int)s[i] == 63)
                { tempLen += 2; }
                else
                { tempLen += 1; }
                try
                { tempString += str.Substring(i, 1); }
                catch
                { break; }
                if (tempLen > len) break;
            }
            //如果截过则加上半个省略号 
            byte[] mybyte = System.Text.Encoding.UTF8.GetBytes(str);
            if (mybyte.Length > len)
                tempString += show;
            tempString = tempString.Replace("&nbsp;", " ");
            tempString = tempString.Replace("&lt;", "<");
            tempString = tempString.Replace("&gt;", ">");
            tempString = tempString.Replace('\n'.ToString(), "<br>");
            return tempString;
        }

        /// <summary>
        /// 截取字符串，汉字两个字节，字母一个字节
        /// </summary>
        /// <param name="str">字符串</param>
        /// <param name="strLength">字符串长度</param>
        /// <returns></returns>
        public static string CutString(this string str, int len, string show = "...")
        {
            return Interruption(str, len, show);
        }

        /// <summary>
        /// 获取左边多少个字符
        /// </summary>
        /// <param name="str"></param>
        /// <param name="len"></param>
        /// <returns></returns>
        public static string Left(this string str, int len)
        {
            if (str == null || len < 1) { return ""; }
            if (len < str.Length)
            { return str.Substring(0, len); }
            else
            { return str; }
        }
        /// <summary>
        /// 获取右边多少个字符
        /// </summary>
        /// <param name="str"></param>
        /// <param name="len"></param>
        /// <returns></returns>
        public static string Right(this string str, int len)
        {
            if (str == null || len < 1) { return ""; }
            if (len < str.Length)
            { return str.Substring(str.Length - len); }
            else
            { return str; }
        }

        /// <summary>
        /// 得到实符串实际长度
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static int Size(this string str)
        {
            byte[] strArray = System.Text.Encoding.UTF8.GetBytes(str);
            int res = strArray.Length;
            return res;
        }

        /// <summary>
        /// 取得html中的图片
        /// </summary>
        /// <param name="HTMLStr"></param>
        /// <returns></returns>
        public static string GetImg(this string text)
        {
            string str = string.Empty;
            Regex r = new Regex(@"<img\s+[^>]*\s*src\s*=\s*([']?)(?<url>\S+)'?[^>]*>", //注意这里的(?<url>\S+)是按正则表达式中的组来处理的，下面的代码中用使用到，也可以更改成其它的HTML标签，以同样的方法获得内容！
            RegexOptions.Compiled);
            Match m = r.Match(text.ToLower());
            if (m.Success)
                str = m.Result("${url}").Replace("\"", "").Replace("'", "");
            return str;
        }
        /// <summary>
        /// 取得html中的所有图片
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static string[] GetImgs(this string text)
        {
            List<string> imgs = new List<string>();
            string pat = @"<img\s+[^>]*\s*src\s*=\s*([']?)(?<url>\S+)'?[^>]*>";
            Regex r = new Regex(pat, RegexOptions.Compiled);
            Match m = r.Match(text.ToLower());
            while (m.Success)
            {
                imgs.Add(m.Result("${url}").Replace("\"", "").Replace("'", ""));
                m = m.NextMatch();
            }
            return imgs.ToArray();
        }
        /// <summary>
        /// 产生随机字符串
        /// </summary>
        /// <returns>字符串位数</returns>
        public static string GetRandom(int length)
        {
            int number;
            char code;
            string checkCode = String.Empty;
            System.Random random = new Random();

            for (int i = 0; i < length + 1; i++)
            {
                number = random.Next();
                if (number % 2 == 0)
                    code = (char)('0' + (char)(number % 10));
                else
                    code = (char)('A' + (char)(number % 26));
                checkCode += code.ToString();
            }
            return checkCode;
        }

        /// <summary>
        /// 字符串是否包含标点符号(不包括_下画线)
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static bool InPunctuation(this string str)
        {
            foreach (char c in str.ToCharArray())
            {
                if (char.IsPunctuation(c) && c != '_')
                    return true;
            }
            return false;

        }
        /// <summary>
        /// 去除字符串标点符号和空字符
        /// </summary>
        /// <param name="sourceString"></param>
        /// <returns></returns>
        public static string RemovePunctuationOrEmpty(this string str)
        {
            StringBuilder NewString = new StringBuilder(str.Length);
            char[] charArr = str.ToCharArray();
            foreach (char symbols in charArr)
            {
                if (!char.IsPunctuation(symbols) && !char.IsWhiteSpace(symbols))
                {
                    NewString.Append(symbols);
                }
            }
            return NewString.ToString();
        }
        /// <summary>
        /// 返回带星期的日期格式
        /// </summary>
        /// <param name="date"></param>
        /// <returns></returns>
        public static string ToDateWeekString(this DateTime date)
        {
            string week = string.Empty;
            switch (date.DayOfWeek)
            {
                case DayOfWeek.Friday: week = "五"; break;
                case DayOfWeek.Monday: week = "一"; break;
                case DayOfWeek.Saturday: week = "六"; break;
                case DayOfWeek.Sunday: week = "日"; break;
                case DayOfWeek.Thursday: week = "四"; break;
                case DayOfWeek.Tuesday: week = "二"; break;
                case DayOfWeek.Wednesday: week = "三"; break;
            }
            return date.ToString("yyyy年M月d日 ") + "星期" + week;
        }
        /// <summary>
        /// 返回带星期的日期时间格式
        /// </summary>
        /// <param name="date"></param>
        /// <returns></returns>
        public static string ToDateTimeWeekString(this DateTime date)
        {
            string week = string.Empty;
            switch (date.DayOfWeek)
            {
                case DayOfWeek.Friday: week = "五"; break;
                case DayOfWeek.Monday: week = "一"; break;
                case DayOfWeek.Saturday: week = "六"; break;
                case DayOfWeek.Sunday: week = "日"; break;
                case DayOfWeek.Thursday: week = "四"; break;
                case DayOfWeek.Tuesday: week = "二"; break;
                case DayOfWeek.Wednesday: week = "三"; break;
            }
            return date.ToString("yyyy年M月d日H时m分") + " 星期" + week;
        }

        /// <summary>
        /// 使用CDATA的方式将value保存在指定的element中
        /// </summary>
        /// <param name="element"></param>
        /// <param name="value"></param>
        public static void SetCDataValue(this System.Xml.Linq.XElement element, string value)
        {
            element.RemoveNodes();
            element.Add(new System.Xml.Linq.XCData(value));
        }

        /// <summary>
        /// 返回一个值，该值指示指定的 System.String 对象是否出现在此字符串中。
        /// </summary>
        /// <param name="source"></param>
        /// <param name="value">要搜寻的字符串。</param>
        /// <param name="comparisonType">指定搜索规则的枚举值之一</param>
        /// <returns>如果 value 参数出现在此字符串中，或者 value 为空字符串 ("")，则为 true；否则为 false</returns>
        public static bool Contains(this string source, string value, StringComparison comparisonType)
        {
            if (source == null || value == null) { return false; }
            if (value == "") { return true; }
            return (source.IndexOf(value, comparisonType) >= 0);
        }

        /// <summary>
        /// 通过使用默认的相等或字符比较器确定序列是否包含指定的元素。
        /// </summary>
        /// <param name="source">要在其中定位某个值的序列。</param>
        /// <param name="value">要在序列中定位的值。</param>
        /// <param name="comparisonType">指定搜索规则的枚举值之一</param>
        /// <exception cref="System.ArgumentNullException">source 为 null</exception>
        /// <returns>如果源序列包含具有指定值的元素，则为 true；否则为 false。</returns>
        public static bool Contains(this string[] source, string value, StringComparison comparisonType)
        {
            return System.Linq.Enumerable.Contains(source, value, new CompareText(comparisonType));
        }
        private class CompareText : IEqualityComparer<string>
        {
            private StringComparison ComparisonType { get; set; }
            public int GetHashCode(string t) { return t.GetHashCode(); }
            public CompareText(StringComparison comparisonType) { this.ComparisonType = comparisonType; }
            public bool Equals(string x, string y)
            {
                if (x == y) { return true; }
                if (x == null || y == null) { return false; }
                else { return x.Equals(y, ComparisonType); }
            }
        }

        public static void TriggerEvent<TArgs>(this object sender, Delegate handler, TArgs args)
        where TArgs : EventArgs
        {
            if (handler != null)
            {
                handler.DynamicInvoke(sender, args);
            }
        }

        public static void TriggerEvent(this object sender, EventHandler handler)
        {
            handler?.Invoke(sender, EventArgs.Empty);
        }

        public static void ForEach<T>(this IEnumerable<T> source, Action<T> action)
        {
            if (source == null)
            {
                throw new ArgumentNullException("source");
            }

            if (action == null)
            {
                throw new ArgumentNullException("action");
            }

            foreach (T element in source)
            {
                action(element);
            }
        }


        public static void Catch(this Action action, Action<Exception> cleanup)
        {
            if (action == null)
            {
                return;
            }

            try
            {
                var task = Task.Run(action);
                task.Wait();
            }
            catch (AggregateException e)
            {
                if (cleanup != null)
                {
                    Catch(() => cleanup(e.InnerException),
                        default(Action<Exception>));
                }
            }
        }

        public static async Task CatchAsync(this Action action, Action<Exception> cleanup)
        {
            if (action == null)
            {
                return;
            }

            try
            {
                await Task.Run(() => action.Catch(cleanup));
            }
            catch (Exception)
            {
                //
            }
        }        

        public static object GetDefaultValue(Type t)
        {
            if (t.IsByRef)
                return Activator.CreateInstance(t);

            return null;
        }

        public static int GetUnixTime(this DateTime date, bool useUtc = true)
        {
            return useUtc
                ? Convert.ToInt32((date - new DateTime(1970, 1, 1, 0, 0, 0, 0).ToUniversalTime().AddHours(8)).TotalSeconds)
                : Convert.ToInt32((date - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalSeconds);
        }

        public static long GetUnixLongTime(this DateTime date, bool useUtc = true)
        {
            return useUtc
                ? Convert.ToInt64((date - new DateTime(1970, 1, 1, 0, 0, 0, 0).ToUniversalTime().AddHours(8)).TotalMilliseconds)
                : Convert.ToInt64((date - new DateTime(1970, 1, 1, 0, 0, 0, 0)).TotalMilliseconds);
        }

        public static DateTime ToDateTime(this long stamp, bool useUtc = true)
        {
            return useUtc
                ? new DateTime(1970, 1, 1, 0, 0, 0, 0).ToUniversalTime().AddHours(8).AddMilliseconds(stamp).ToLocalTime()
                : new DateTime(1970, 1, 1, 0, 0, 0, 0).AddMilliseconds(stamp);
        }

        public static DateTime GetUtcNowTime(this DateTime date)
        {
            return DateTime.Now.ToUniversalTime().AddHours(8);
        }

        public static DateTime ToDateTime(this int stamp, bool useUtc = true)
        {
            return useUtc
                ? new DateTime(1970, 1, 1, 0, 0, 0, 0).ToUniversalTime().AddHours(8).AddSeconds(stamp).ToLocalTime()
                : new DateTime(1970, 1, 1, 0, 0, 0, 0).AddSeconds(stamp);
        }


        public static void MergeInto(this JContainer left, JToken right)
        {
            foreach (var rightChild in right.Children<JProperty>())
            {
                var rightChildProperty = rightChild;
                var leftProperty = left.Value<JToken>(rightChildProperty.Name);

                if (leftProperty == null)
                {
                    // no matching property, just add 
                    left.Add(rightChild);
                }
                else
                {
                    if (leftProperty.Type != JTokenType.Array && leftProperty.Type != JTokenType.Object)
                    {
                        // replace value
                        var leftParent = (JProperty)leftProperty.Parent;
                        leftParent.Value = rightChildProperty.Value;
                    }
                    else
                    {
                        if (leftProperty.Type == JTokenType.Object)
                        {
                            // recurse object
                            MergeInto(leftProperty as JObject, rightChildProperty.Value);
                        }
                        else if(leftProperty.Type == JTokenType.Array)
                        {
                            MergeArray(leftProperty, rightChildProperty.Value);
                        }
                    }
                }
            }
        }

        private static void MergeArray(JToken leftObject, JToken value)
        {
            var targetItems = leftObject as JArray;
            var sourceItems = value as JArray;

            for (int i = 0; i < targetItems.Count; i++)
            {
                if (sourceItems.Count <= i)
                {
                    LoggerManager.Logger.Warn($"Merge {leftObject.Parent.Path} mismatch.");
                    break;
                }

                MergeInto(targetItems[i] as JContainer, sourceItems[i] as JToken);
            }
        }

        public static JToken Merge(this JToken left, JToken right, bool clone = true)
        {
            if (left.Type != JTokenType.Object || right.Type != JTokenType.Object)
            {
                throw new Exception("Merge can be only used for JObject.");
            }

            var leftClone = clone ? (JContainer)left.DeepClone() : (JContainer)left;

            MergeInto(leftClone, right);

            return leftClone;
        }

        public static void RemoveProperties(this JToken obj, params string[] properties)
        {
            if (properties?.Length > 0)
            {
                if (obj is JObject)
                {
                    (obj as JObject).Properties().Where(p => properties.Contains(p.Name)).ToList().ForEach(p => p.Remove());
                }
                else if (obj is JArray)
                {
                    foreach (var item in obj as JArray)
                    {
                        item.RemoveProperties(properties);
                    }
                }
            }
        }

        public static void KeepProperties(this JToken obj, params string[] properties)
        {
            if (properties?.Length > 0)
            {
                if (obj is JObject)
                {
                    (obj as JObject).Properties().Where(p => !properties.Contains(p.Name)).ToList().ForEach(p => p.Remove());
                }
                else if (obj is JArray)
                {
                    foreach (var item in obj as JArray)
                    {
                        item.KeepProperties(properties);
                    }
                }
            }
        }

        //MD5加密
        public static string EncryptWithMD5(this string source, string salt)
        {
            string md5Password = Encrypt(source);

            if (salt != null && salt != "0")
            {
                md5Password += salt;
                return Encrypt(md5Password);
            }

            return md5Password;
        }


        private static string Encrypt(string source)
        {
            MD5 md5 = MD5.Create();
            byte[] data = Encoding.UTF8.GetBytes(source);
            byte[] md5Data = md5.ComputeHash(data);
            string str = "";

            for (int i = 0; i < md5Data.Length; i++)
            {
                str += md5Data[i].ToString("x").PadLeft(2, '0');
            }

            return str;
        }

        public static Task<HttpResponseMessage> PostAsJsonAsync<T>(
            this HttpClient httpClient, string url, T data)
        {
            var dataAsString = JsonConvert.SerializeObject(data);
            var content = new StringContent(dataAsString);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            return httpClient.PostAsync(url, content);
        }

        public static Task<HttpResponseMessage> PutAsJsonAsync<T>(
            this HttpClient httpClient, string url, T data)
        {
            var dataAsString = JsonConvert.SerializeObject(data);
            var content = new StringContent(dataAsString);
            content.Headers.ContentType = new MediaTypeHeaderValue("application/json");
            return httpClient.PutAsync(url, content);
        }

        public static async Task<T> ReadAsJsonAsync<T>(this HttpContent content)
        {
            var dataAsString = await content.ReadAsStringAsync();
            return JsonConvert.DeserializeObject<T>(dataAsString);
        }


        /// <summary>  
        /// 数字转换为指定的进制形式字符串  
        /// </summary>  
        /// <param name="source"></param>  
        /// <returns></returns>  
        public static string ConvertWith(this long number, string source)
        {
            List<string> result = new List<string>();
            long t = number;

            while (t > 0)
            {
                var mod = t % source.Length;
                t = Math.Abs(t / source.Length);
                var character = source[Convert.ToInt32(mod)].ToString();
                result.Insert(0, character);
            }

            return string.Join("", result.ToArray());
        }

        /// <summary>  
        /// 指定字符串转换为指定进制的数字形式  
        /// </summary>  
        /// <param name="source"></param>  
        /// <returns></returns>  
        public static long RevertWith(this string str, string source)
        {
            long result = 0;
            int j = 0;
            foreach (var ch in new string(str.ToCharArray().Reverse().ToArray()))
            {
                if (source.Contains(ch))
                {
                    result += source.IndexOf(ch) * ((long)Math.Pow(source.Length, j));
                    j++;
                }
            }
            return result;
        }

        public static string GetRemoteIpAddressBehindProxy(this HttpContext context)
        {
            if (context.Request.Headers.TryGetValue("X-Forwarded-For", out var value))
            {
                return value[0].Split(',')[0];
            }
            else if (context.Request.Headers.TryGetValue("X-Real-Ip", out value))
            {
                return value[0];
            }
            else
            {
                return context.Connection.RemoteIpAddress.MapToIPv4().ToString();
            }
        }

        public static void CopyPropertiesTo<S, D>(this S source, D dest)
        {
            var sourceProps = typeof(S).GetProperties().Where(x => x.CanRead).ToList();
            var destProps = typeof(D).GetProperties().Where(x => x.CanWrite).ToList();

            foreach (var sourceProp in sourceProps)
            {
                if (destProps.Any(x => x.Name == sourceProp.Name))
                {
                    var p = destProps.First(x => x.Name == sourceProp.Name);
                    if (p.CanWrite) // check if the property can be set or no.
                    { 
                        p.SetValue(dest, sourceProp.GetValue(source, null), null);
                    }
                }
            }
        }

        #region 查询中去除空参数

        public static void RemoveEmptyValues(this JObject param, params object[] additionalValues)
        {
            var dict = param as IDictionary<string, JToken>;
            if (dict == null) return;

            var removableKeys = dict.Keys.Where(k => param[k] == null || param[k].ToString() == string.Empty || additionalValues?.Contains(param[k]) == true).ToArray();

            foreach (var item in removableKeys)
            {
                param.Remove(item);
            }
        }

        #endregion
    }
}